########################################################################
#
# File Name:            Uri.py
#
# Documentation:        http://docs.ftsuite.com/4Suite/Uri.py.html
#
"""
WWW: http://4suite.com/4Suite         e-mail: support@4suite.com

Copyright (c) 2000 Fourthought Inc., USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

import string, urlparse, urllib, os
from Ft.Lib import Uuid

class BaseUriResolver:
    def normalize(self, uri, base=''):
        "Default action is to simply resolve to an absolute URL"
        (scheme, netloc, path, params, query, fragment) = urlparse.urlparse(uri)
        if not scheme:
            uri = urlparse.urljoin(base, uri)
        if path:
            paths = os.path.split(path)
            path = apply(os.path.join,paths[:-1])
        baseUri = urlparse.urlunparse((scheme, netloc, path, params, query, fragment))
        if baseUri and not baseUri[-1] == '/':
            baseUri = baseUri + '/'
        return uri, baseUri

    def resolve(self, uri, base=''):
        uri = self.normalize(uri, base)[0]

        expandedUri = urlparse.urlparse(uri)

        #Hack cause urllib doesn't work well on Windows
        if os.path.exists(expandedUri[2]):
            stream = open(expandedUri[2])
        else:
            stream = urllib.urlopen(uri)
        return stream

    def generate(self, hint=None):
        """
        The hint is an object that helps decide what to generate
        Default action is to generate a UUID URN
        """
        return 'urn:uuid:'+Uuid.UuidAsString(Uuid.GenerateUuid())


BASIC_RESOLVER = BaseUriResolver()

