############################################################################
#
# File Name: 		CollectionBase.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/CollectionBase.py.html
#
"""
Base collection class.
WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 1999 FourThought LLC, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

from Ft.Ods.Collections import Iterator
from Ft.Ods.Collections import BidirectionalIterator


class CollectionBase:
    #Exception classes
    class InvalidCollectionType:
        pass

    class ElementNotFound:
        def __init__(self,element):
            self.element = element

    def __init__(self):
        pass

    def cardinality(self):
        return self.__dict__['_size']

    def is_empty(self):
        return self.cardinality() == 0

    def is_ordered(self):
        return 0

    def allows_duplicates(self):
        return 0

    def contains_element(self, e):
        return self._4ods_getIndex(e) != -1

    def insert_element(self,e):
        self._4ods_insertElementByIndex(e,-1)

    def remove_element(self,e):
        index = self._4ods_getIndex(e)
        if index == -1:
            raise CollectionBase.ElementNotFound()
        self._4ods_removeElementByIndex(index)

    def copy(self,deep=1):
        initArgs = ()
        if hasattr(self,'__getinitargs__'):
            initArgs = self.__getinitargs__()
           
        other = apply(self.__class__,initArgs)

        if deep:
            vals = self._4ods_getContents()
            for v in vals:
               other.insert_element(v)
        return other

    def create_iterator(self, stable):
        #No stable iterators until we can examine the effects of mutation
        #in more detail
        return Iterator.Iterator(self, stable)

    def create_bidirectional_iterator(self, stable):
        #No stable BidirectionalIterators until we can examine the effects
        #of mutation in more detail
        return BidirectionalIterator.BidirectionalIterator(self, stable)

    def select_element(self,OQL_predicate):
        raise FtodsNotSupportedException("select_element")

    def select(self,OQL_predicate):
        raise FtodsNotSupportedException("select")

    def query(self,OQL_predicate):
        raise FtodsNotSupportedException("select_element")

    def exists_element(self,OQL_predicate):
        raise FtodsNotSupportedException("select_element")


    #Some helpers
    def _4ods_isObject(self,obj):
        from Ft.Ods import Object
        return isinstance(obj,Object.Object)
    def _4ods_isLiteral(self,lit):
        from Ft.Ods import Literal
        return isinstance(lit,Literal.Literal)
    def _4ods_isRepositoryObject(self,ro):
        from Ft.Ods.MetaData import RepositoryObject
        return isinstance(ro,RepositoryObject.RepositoryObject)


    #The pythonic interface
    def __cmp__(self, other):
        if isinstance(other, CollectionBase):
            return cmp(self._4ods_getContents(), other._4ods_getContents())
        else:
            return cmp(self._4ods_getContents(), other)
    def __len__(self): return self.cardinality()
    def __getitem__(self, i):
        if i >= self.cardinality():
            raise IndexError(i)
        if i < 0:
            i = self.cardinality() + i
        return self._4ods_retrieveElementByIndex(i)
    #def __setitem__(self, i, item): self._4ods_insertElementByIndex(item,i)
    #def __delitem__(self, i): self._4ods_removeElementByIndex(i)
    def __getslice__(self, i, j):
        i = max(i, 0); j = min(j, self.cardinality())
        #FIXME, what are the implecations of two collections with the same set of data?
        rt = self.copy(deep=0)
        rt._4ods_initialize(j-i,self._4ods_getSubType(),self._4ods_getContents()[i:j])
        return rt
    #def __setslice__(self, i, j, other):
    #    i = max(i, 0); j = max(j, 0)
    #    if isinstance(other, CollectionBase):
    #        otherData = list(other._4ods_getContents())
    #        ourData = list(self._4ods_getContents())
    #        ourData[i:j] = otherData
    #        self._4ods_clearContents()
    #        self._4ods_initialize(len(ourData),self._4ods_getSubType(),ourData)
    #    elif type(other) == type([]):
    #        ourData = list(self._4ods_getContents())
    #        ourData[i:j] = other
    #        self._4ods_clearContents()
    #        self._4ods_initialize(len(ourData),self._4ods_getSubType(),ourData)
    #    else:
    #        ourData = list(self._4ods_getContents())
    #        ourData[i:j] = list(other)
    #        self._4ods_clearContents()
    #        self._4ods_initialize(len(ourData),self._4ods_getSubType(),ourData)

    #    self._4ods_setModified()

    #def __delslice__(self, i, j):
    #    i = max(i, 0); j = max(j, 0)
    #    ourData = list(self._4ods_getContents())
    #    del ourData[i:j]
    #    self._4ods_clearContents()
    #    self._4ods_initialize(ourData)
    #    self._4ods_setModified()

    #def __add__(self, other):
    #    rt = self.copy()
    #    if isinstance(other, CollectionBase):
    #        rt._4ods_initialize(list(self._4ods_getContents()) + list(other._4ods_getContents()))
    #    elif type(other) ==  type([]):
    #        rt._4ods_initialize(list(self._4ods_getContents()) + other)
    #    else:
    #        rt._4ods_initialize(list(self._4ods_getContents()) + list(other))
    #    return rt

    #def __radd__(self, other):
    #    rt = self.copy(deep = 0)
    #    if isinstance(other, CollectionBase):
    #        rt._4ods_initialize(list(other._4ods_getContentes()) + list(self._4ods_getContents()))
    #    elif type(other) ==  type([]):
    #        rt._4ods_initialize(other + list(self._4ods_getContents()))
    #    else:
    #        rt._4ods_initialize(list(other) + list(self._4ods_getContents()))

    #def __mul__(self, n):
    #    rt = self.copy(deep = 0)
    #    rt._4ods_initialize(self._4ods_getContents()*n)
    #    return rt
    #__rmul__ = __mul__

    def append(self, item): self._4ods_insertElementByIndex(item,-1)

    def insert(self, i, item): self._4ods_insertElementByIndex(item,i)

    def pop(self, i=-1): return self._4ods_retrieveElementByIndex(i)

    def remove(self, item): self.remove_element(item)

    def count(self, item): return list(self._4ods_getContents()).count(item)

    def index(self, item): return list(self._4ods_getContents()).index(item)

    #def reverse(self):
    #    data = list(self._4ods_getContents())
    #    data.reverse()
    #    self._4ods_clearContents()
    #    self._4ods_initialize(data)
    #    self._4ods_setModified()

    #def sort(self, *args): 
    #    data = list(self._4ods_getContents())
    #    apply(data.sort, args)
    #    self._4ods_clearContents()
    #    self._4ods_initialize(data)
    #    self._4ods_setModified()

    #def extend(self, other):
    #    data = list(self._4ods_getContents())
    #    if isinstance(other, CollectionBase):
    #        data.extend(list(other._4ods_getContents()))
    #    else:
    #        data.extend(other)
    #    self._4ods_clearContents()
    #    self._4ods_initialize(data)
    #    self._4ods_setModified()




    ############################
    #Functions to be overriden by derived classes
    ############################
    def _4ods_getContents(self):
        raise "Must Override"

    def _4ods_getIndex(self,element):
        raise "Must Override"

    def _4ods_insertElementByIndex(self,element,index):
        raise "Must Override"

    def _4ods_removeElementByIndex(self,index):
        raise "Must OVerrid"

    def _4ods_retrieveElementByIndex(self,index):
        raise "Must Override"

    def _4ods_countOccurrences(self):
        raise "Must be overriden"
