

from Ft.Ods import Database
from Ft.Ods.StorageManager.Adapters import Constants

import types

g_toOdmgType = {types.StringType:Constants.Types.STRING,
                types.InstanceType:Constants.Types.OBJECT,
                types.IntType:Constants.Types.SIGNED_SHORT,
                types.FloatType:Constants.Types.DOUBLE,
                }

#FIXME improve with a func to check loaded

class CollectionOfValues:

    def __init__(self):
        self.__dict__['_values'] = []
        self.__dict__['_loaded'] = 0
        self.__dict__['_size'] = 0
        self.__dict__['_subType'] = -1

        #These dictionaries map our indexs to those in the database
        self.__dict__['_dbMapping'] = {}
        self.__dict__['_inverseDbMapping'] = {}
        self.__dict__['_dbSize'] = 0

        #A list of changes to tell the DB about
        self.__dict__['_dbChangeLog'] = []

    def _4ods_initialize(self,siz,subType,values = None):

        self.__dict__['_size'] = siz
        if values == None:
            self.__dict__['_values'] = [None] * siz
        else:
            self.__dict__['_values'] = values
        self.__dict__['_subType'] = subType
        self.__dict__['_dbSize'] = siz
        self.__dict__['_dbChangeLog'] = []
        for ctr in range(siz):
            self.__dict__['_dbMapping'][ctr] = ctr
            self.__dict__['_inverseDbMapping'][ctr] = ctr
            

    def _4ods_setTransaction(self,tx,unregister=0):
        from Ft.Ods import Collection
        Collection.Collection._4ods_setTransaction(self,tx,unregister)
        self.__dict__['_dbChangeLog'] = []
        for ctr in range(self._size):
            self.__dict__['_dbMapping'][ctr] = ctr
            self.__dict__['_inverseDbMapping'][ctr] = ctr
        

    def _4ods_clearContents(self):
        self.__dict__['_values'] = []
        self.__dict__['_size'] = 0
        self.__dict__['_dbMapping'] = {}
        self.__dict__['_inverseDbMapping'] = {}
        self.__dict__['_dbChangeLog'] = []
        self.__dict__['_dbSize'] = 0

    def _4ods_getContents(self):
        self._4ods_loadData()
        return map(lambda x,f=self._4ods_objectExternalPrepare:f(x),self.__dict__['_values'])

    def _4ods_loadData(self):
        if not self._4ods_getCid():
            return
        if self.__dict__['_loaded']:
            return

        self.__dict__['_loaded'] = 1

        data = self.__dict__['_db']._4ods_loadCollectionData(self)

        ctr = 0
        for item in data:
            listIndex = self.__dict__['_inverseDbMapping'][ctr]
            if self.__dict__['_values'][listIndex] == None:
                self.__dict__['_values'][listIndex] = item
            ctr = ctr + 1

    #Hooks for derived classes
    def _4ods_objectExternalPrepare(self,obj):
        return obj

    def _4ods_objectInternalPrepare(self,obj):
        return obj


    #For specific collection classes
    def _4ods_getIndex(self,element):
        self._4ods_loadData()
        if element in self.__dict__['_values']:
            return self.__dict__['_values'].index(element)
        conts = self._4ods_getContents()
        if element in conts:
            return conts.index(element)
        return -1

    def _4ods_insertElementByIndex(self,element,index):
        if index == -1:
            index = self.__dict__['_size']
        self.__dict__['_values'].insert(index,element)
        if self.__dict__['_size'] == index:
            op = Constants.CollectionChanges.APPEND
        else:
            op = Constants.CollectionChanges.INSERT


        #Modify the mappings and record the change for the DB

        #Add entry in dbMapping (we'll set it later)
        self.__dict__['_dbMapping'][self.__dict__['_size']] = -1

        #Shift (from the back) to be the previous value up to the index
        keys = self.__dict__['_dbMapping'].keys()
        keys.sort()
        keys = keys[index:-1]
        keys.reverse()

        for key in keys:
            self.__dict__['_dbMapping'][key+1] = self.__dict__['_dbMapping'][key]

        #Clear out what is currently at the index
        self.__dict__['_dbMapping'][index] = -1

        self.__dict__['_size'] = self.__dict__['_size'] + 1

        self._4ods_synchInverseMapping()

        #Record the change
        self.__dict__['_dbChangeLog'].append((op,element,index))

        #Set up te subtype
        self.__dict__['_subType'] = g_toOdmgType[type(element)]


        
    def _4ods_removeElementByIndex(self,index):
        del self.__dict__['_values'][index]
        
        if index == -1:
            index = self.__dict__['_size']

        if self.__dict__['_size'] == index:
            op = Constants.CollectionChanges.REMOVE_END
        else:
            op = Constants.CollectionChanges.REMOVE


        self.__dict__['_size'] = self.__dict__['_size'] - 1

        #Shift (from the index) to be the next value up to the index
        keys = self.__dict__['_dbMapping'].keys()
        keys.sort()

        for key in keys[:-1]:
            if key >= index:
                self.__dict__['_dbMapping'][key] = self.__dict__['_dbMapping'][key+1]

        #Remove the last entry
        del self.__dict__['_dbMapping'][self.__dict__['_size']]

        self._4ods_synchInverseMapping()


        #Record the change
        self.__dict__['_dbChangeLog'].append((op,index))



    def _4ods_synchInverseMapping(self):
        self.__dict__['_inverseDbMapping'] = {}
        for ctr in range(self.__dict__['_dbSize']):
            self.__dict__['_inverseDbMapping'][ctr] = -1

        for key in self.__dict__['_dbMapping'].keys():
            if self.__dict__['_dbMapping'][key]>= 0:
                self.__dict__['_inverseDbMapping'][self.__dict__['_dbMapping'][key]] = key

    def _4ods_retrieveElementByIndex(self,index):
        if self.__dict__['_values'][index] == None:
            self.__dict__['_values'][index] = self.__dict__['_db']._4ods_loadCollectionData(self,self.__dict__['_dbMapping'][index])[0]
        return self._4ods_objectExternalPrepare(self.__dict__['_values'][index])

    def _4ods_countOccurrences(self,element):
        return self._4ods_getContents().count(element)


    def _4ods_getChanges(self):
        rt = map(lambda x,f=self._4ods_objectInternalPrepare,i=[Constants.CollectionChanges.INSERT,Constants.CollectionChanges.APPEND]: (x[0] in i and (x[0],f(x[1]),x[2])) or x,self.__dict__['_dbChangeLog'])
        self.__dict__['_dbChangeLog'] = []
        return rt

    def _4ods_isModified(self):
        return len(self.__dict__['_dbChangeLog'])

    def _4ods_getSubType(self):
        if self.__dict__['_subType'] == -1:
            test = self.__dict__['_values'][0]
            self.__dict__['_subType'] = g_toOdmgType[type(test)]
        return self.__dict__['_subType']
