############################################################################
#
# File Name: 		ListBase.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/List.py.html
#
"""
Lists: ordered collections.
WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 1999 FourThought LLC, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

from Ft.Ods.Collections import CollectionBase
from Ft.Ods.StorageManager.Adapters import Constants


class ListBase(CollectionBase.CollectionBase):

    #Exception classes
    class InvalidIndex:
        def __init__(self,index):
            self.index = index

    def __init__(self):
        self.__dict__['_kind'] = 'List'

    def is_ordered(self):
        return 1

    def allows_duplicates(self):
        return 1

    #FIXME No value support
    def remove_element_at(self, index):
        if index < 0:
            raise ListBase.InvalidIndex(index)
        if index >= self.cardinality():
            raise ListBase.InvalidIndex(index)
        self._4ods_removeElementByIndex(index)

    def retrieve_element_at(self, index):
        if index < 0:
            raise ListBase.InvalidIndex(index)
        if index >= self.cardinality():
            raise ListBase.InvalidIndex(index)
        return self._4ods_retrieveElementByIndex(index)

    def replace_element_at(self, element, index):
        if index < 0:
            raise ListBase.InvalidIndex(index)
        if index >= self.cardinality():
            raise ListBase.InvalidIndex(index)
        self._4ods_removeElementByIndex(index)
        self._4ods_insertElementByIndex(element,index)

    def insert_element_after(self, element, index):
        if index < 0:
            raise ListBase.InvalidIndex(index)
        if index >= self.cardinality():
            raise ListBase.InvalidIndex(index)
        self._4ods_insertElementByIndex(element,index+1)

    def insert_element_before(self, element, index):
        if index < 0:
            raise ListBase.InvalidIndex(index)
        if index >= self.cardinality():
            raise ListBase.InvalidIndex(index)
        self._4ods_insertElementByIndex(element,index)

    def insert_element_first(self, obj):
        self._4ods_insertElementByIndex(obj,0)

    def insert_element_last(self, obj):
        self._4ods_insertElementByIndex(obj,-1)

    def remove_first_element(self):
        if not self.cardinality():
            raise CollectionBase.CollectionBase.ElementNotFound(None)
        self._4ods_removeElementByIndex(0)

    def remove_last_element(self):
        if not self.cardinality():
            raise CollectionBase.CollectionBase.ElementNotFound(None)
        self._4ods_removeElementByIndex(-1)

    def retrieve_first_element(self):
        if not self.cardinality():
            raise CollectionBase.CollectionBase.ElementNotFound(None)
        return self._4ods_retrieveElementByIndex(0)

    def retrieve_last_element(self):
        if not self.cardinality():
            raise CollectionBase.CollectionBase.ElementNotFound(None)
        return self._4ods_retrieveElementByIndex(-1)



    def concat(self, otherList):
        result = self.copy(deep=0)
        result.append(self)
        result.append(otherList)
        return result

    def append(self, otherList):
        if not isinstance(otherList,ListBase):
            return CollectionBase.CollectionBase.append(self,otherList)
        for e in otherList._4ods_getContents():
            self.insert_element_last(e)


    insert_element = insert_element_last
    collectionType = Constants.CollectionTypes.LIST
