############################################################################
#
# File Name: 		SetBase.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/SetBase.py.html
#
"""
Sets: collections disallowing duplicates.
WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 1999 FourThought LLC, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

from Ft.Ods.StorageManager.Adapters import Constants


from Ft.Ods.Collections import CollectionBase

class SetBase(CollectionBase.CollectionBase):
    def __init__(self):
        CollectionBase.CollectionBase.__init__(self)
        self.__dict__['_kind'] = 'Set'

    def is_ordered(self):
        return 0

    def allows_duplicates(self):
        return 0

    #FIXME No supportfor the value element


    def create_union(self, other_set):
        result = self.copy(deep=0)
        other_py_list = other_set._4ods_getContents()
        our_py_list = self._4ods_getContents()

        for item in other_py_list:
            if item in our_py_list:
                result.insert_element(item)
            
        return result

    def create_intersection(self, other_set):
        result = self.copy(deep=0)
        vals = list(other_set._4ods_getContents())
        our_py_list = self._4ods_getContents()

        for item in our_py_list:
            if not item in vals:
                vals.append(item)
        for v in vals:
            result.insert_element(v)
        return result

    def create_difference(self, other_set):
        result_list = []
        result = self.copy(deep=0)
        other_py_list = other_set._4ods_getContents()
        our_py_list = self._4ods_getContents()
        for item in other_py_list:
            if not item in our_py_list:
                result_list.append(item)
        for item in our_py_list:
            if not item in other_py_list:
                result_list.append(item)
        for item in result_list:
            result.insert_element(item)
        return result

    def is_subset_of(self, other_set):
        for item in self._4ods_getContents():
            if not other_set.contains_element(item):
                return 0
        return 1

    def is_superset_of(self, other_set):
        return other_set.is_subset_of(self)

    def is_proper_subset_of(self,other_set):
        return self.is_subset_of(other_set) and (self.cardinality() != other_set.cardinality())

    def is_proper_superset_of(self,other_set):
        return other_set.is_proper_subset_of(self)


    def insert_element(self,element):
        if not self.contains_element(element):
            CollectionBase.CollectionBase.insert_element(self,element)

    collectionType = Constants.CollectionTypes.SET
