########################################################################
#
# File Name: 	        Class.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/Class.py.html
#
"""
Implements the Class meta-data interface.
WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 1999 FourThought LLC, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

from Ft.Ods.MetaData import Interface
from Ft.Ods.MetaData import RepositoryObject
from Ft.Ods.Collections import LiteralListOfValues
from Ft.Ods.Collections import LiteralSetOfRepositoryObjects


from Ft.Ods.StorageManager.Adapters import Constants


class Class(Interface.Interface):
    def __init__(self):
        Interface.Interface.__init__(self,RepositoryObject.MetaKind.mk_class)

    #relationship extender
    def form_extender(self, extender, inverse=1):
        self._4ods_formRelationship('extender',Class,'extensions','add',extender,inverse)

    def drop_extender(self, extender, inverse=1):
        self._4ods_dropRelationship('extender','extensions','remove',extender,inverse)

    #relationship extensions
    def form_extensions(self, extensions, inverse=1):
        self.add_extensions(extensions,inverse)

    def drop_extensions(self, extensions, inverse=1):
        self.remove_extensions(extensions,inverse)

    def add_extensions(self,target,inverse=1):
        self._4dods_addRelationship('extensions',Class,'extender','form',target,inverse)

    def remove_extensions(self,target,inverse=1):
        self._4ods_removeRelationship('extensions','extender','drop',target,inverse)


    def _4ods_getOdl(self):
        extends = ''
        if self.extender:
            extends = 'extends %s' % self.extender.absolute_name()
        extents = ''
        if self.extents.cardinality():
            extents= '(extent ' + self.extents._4ods_getContents()[0] + ')'
        st = 'class %s %s' % (self.name,extends)
        for i in self.inherits._4ods_getContents():
            st = st + " : %s," % i.absolute_name()
        st = st[:-1]
        if extents:
            st = st + ' ' + extents
        st = st + " {\n"

        for e in self.defines._4ods_getContents():
            st = st + "  %s\n" % e._4ods_getOdl()
        st = st + " };\n"
        return st

    readOnlyAttributes = Interface.Interface.readOnlyAttributes + ('extents','keys','extender','extensions')
    readWriteAttributes = Interface.Interface.readWriteAttributes 
    relationships = Interface.Interface.relationships + ('extender','extensions')
    _collections = Interface.Interface._collections + ('extents','keys','extensions')

    tupleNames =  Interface.Interface.tupleNames + ('extents',
                                                    'keys',
                                                    'extender',
                                                    'extensions'
                                                    )
    tupleTypes =  Interface.Interface.tupleTypes + (Constants.Types.STRING_LIST,
                                                    Constants.Types.STRING_LIST,
                                                    Constants.Types.OBJECT,
                                                    Constants.Types.OBJECT_SET
                                                    )
        

    localExtents = ()

    collectionMap = Interface.Interface.collectionMap.copy()
    collectionMap.update({'extents':LiteralListOfValues.LiteralListOfValues,
                          'keys':LiteralListOfValues.LiteralListOfValues,
                          'extensions':LiteralSetOfRepositoryObjects.LiteralSetOfRepositoryObjects
                          }
                         )
    
