########################################################################
#
# File Name: 	       Collection.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/Collection.py.html
#
"""
Implements the Collection meta-data interface.
WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 1999 FourThought LLC, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

from Ft.Ods.MetaData import Type
import string

from Ft.Ods.MetaData import RepositoryObject
from Ft.Ods.StorageManager.Adapters import Constants


g_typeMap = {
    Constants.Types.BOOLEAN: {'ck_bag':Constants.Types.BOOLEAN_BAG,
                              'ck_set':Constants.Types.BOOLEAN_SET,
                              'ck_list':Constants.Types.BOOLEAN_LIST
                              },
    Constants.Types.SIGNED_SHORT:   {'ck_bag':Constants.Types.SIGNED_SHORT_BAG,
                                     'ck_set':Constants.Types.SIGNED_SHORT_SET,
                                     'ck_list':Constants.Types.SIGNED_SHORT_LIST
                              },
    Constants.Types.STRING:   {'ck_bag':Constants.Types.STRING_BAG,
                              'ck_set':Constants.Types.STRING_SET,
                              'ck_list':Constants.Types.STRING_LIST
                              },
    Constants.Types.UNSIGNED_SHORT:   {'ck_bag':Constants.Types.UNSIGNED_SHORT_BAG,
                                     'ck_set':Constants.Types.UNSIGNED_SHORT_SET,
                                     'ck_list':Constants.Types.UNSIGNED_SHORT_LIST
                              },
    Constants.Types.SIGNED_LONG:   {'ck_bag':Constants.Types.SIGNED_LONG_BAG,
                                     'ck_set':Constants.Types.SIGNED_LONG_SET,
                                     'ck_list':Constants.Types.SIGNED_LONG_LIST
                              },
    
    Constants.Types.UNSIGNED_LONG:   {'ck_bag':Constants.Types.UNSIGNED_LONG_BAG,
                                      'ck_set':Constants.Types.UNSIGNED_LONG_SET,
                                      'ck_list':Constants.Types.UNSIGNED_LONG_LIST
                                      },
    
    Constants.Types.SIGNED_LONG_LONG:   {'ck_bag':Constants.Types.SIGNED_LONG_LONG_BAG,
                                         'ck_set':Constants.Types.SIGNED_LONG_LONG_SET,
                                         'ck_list':Constants.Types.SIGNED_LONG_LONG_LIST
                                         },
    Constants.Types.FLOAT:   {'ck_bag':Constants.Types.FLOAT_BAG,
                              'ck_set':Constants.Types.FLOAT_SET,
                              'ck_list':Constants.Types.FLOAT_LIST
                              },
    Constants.Types.DOUBLE:   {'ck_bag':Constants.Types.DOUBLE_BAG,
                               'ck_set':Constants.Types.DOUBLE_SET,
                               'ck_list':Constants.Types.DOUBLE_LIST
                               },
    
    Constants.Types.OBJECT: {'ck_list':Constants.Types.OBJECT_LIST,
                             'ck_bag':Constants.Types.OBJECT_BAG,
                             'ck_set':Constants.Types.OBJECT_SET
                             }
    }

class Collection(Type.Type):

    def __init__(self,meta_kind = RepositoryObject.MetaKind.mk_collection,collection_kind=None):
        Type.Type.__init__(self,meta_kind)
        self.__dict__['collection_kind'] = collection_kind

    def form_subtype(self,target,inverse = 1):
        self._4ods_formRelationship('subtype',Type.Type,'collections','add',target,inverse)

    def drop_subtype(self, target, inverse=1):
        self._4ods_dropRelationship('subtype','collections','remove',target,inverse)

    def form_max_size(self,target,inverse = 1):
        self._4ods_formRelationship('max_size',Type.Type,'size_of','form',target,inverse)

    def drop_max_size(self, target, inverse=1):
        self._4ods_dropRelationship('max_size','size_of','drop',target,inverse)



    def _4ods_getOdl(self):
        if self.collection_kind == CollectionKind.ck_list:
            return "List"
        if self.collection_kind == CollectionKind.ck_set:
            return "Set"
        if self.collection_kind == CollectionKind.ck_bag:
            return "Bag"
    def _4ods_getOdmgType(self):
        global g_typeMap
        return g_typeMap[self.subtype._4ods_getOdmgType()][self.collection_kind._n]

    def _4ods_getPythonClass(self,name):
        if name == 'collection_kind':
            return CollectionKind
        return Type.Type._4ods_getPythonClass(self,name)


    readOnlyAttributes = Type.Type.readOnlyAttributes + ('collection_kind',
                                                         'max_size',
                                                         'subtype'
                                                         )
    readWriteAttributes = Type.Type.readWriteAttributes 
    relationships = Type.Type.relationships + ('max_size',
                                               'subtype'
                                               )

    tupleNames =  Type.Type.tupleNames + ('collection_kind',
                                          'max_size',
                                          'subtype'
                                          )
    tupleTypes =  Type.Type.tupleTypes + (Constants.Types.ENUMERATION,
                                          Constants.Types.OBJECT,
                                          Constants.Types.OBJECT
                                          )
                    

    localExtents = ()


    _collection = Type.Type._collections
    collectionMap = Type.Type.collectionMap.copy()



class CollectionKindEnumMember: pass

class CollectionKindEnum:
   def __init__(self):
        self.elements = []

   def lookup(self,st):
       for e in self.elements:
           if e._n == st: return e
       return ''
        
   class ck_list(CollectionKindEnumMember):
       _n = 'ck_list'
       _v = 0
   class ck_array(CollectionKindEnumMember):
       _n = 'ck_array'
       _v = 1
   class ck_bag(CollectionKindEnumMember):
       _n = 'ck_bag'
       _v = 2
   class ck_set(CollectionKindEnumMember):
       _n = 'ck_set'
       _v = 3
   class ck_dictionary(CollectionKindEnumMember):
       _n = 'ck_dictionary'
       _v = 4

CollectionKind = CollectionKindEnum()
CollectionKind.elements.append(CollectionKindEnum.ck_list)
CollectionKind.elements.append(CollectionKindEnum.ck_array)
CollectionKind.elements.append(CollectionKindEnum.ck_bag)
CollectionKind.elements.append(CollectionKindEnum.ck_set)
CollectionKind.elements.append(CollectionKindEnum.ck_dictionary)
