########################################################################
#
# File Name: 	       Constant.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/Constant.py.html
#
"""
Implements the Constant meta-data interface.
WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 1999 FourThought LLC, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

from Ft.Ods.MetaData import MetaObject

from Ft.Ods.MetaData import ConstOperand
from Ft.Ods.MetaData import RepositoryObject
from Ft.Ods.Collections import LiteralSetOfRepositoryObjects
from Ft.Ods.StorageManager.Adapters import Constants

class Constant(MetaObject.MetaObject):
    def __init__(self):
        MetaObject.MetaObject.__init__(self,RepositoryObject.MetaKind.mk_constant)



    def value(self):
        if self.the_Value:
            return self.the_Value.value()
        return None

    def form_the_Value(self,target,inverse=1):
        from Ft.Ods.MetaData import Operand
        self._4ods_formRelationship('the_Value',Operand.Operand,'value_of','form',target,inverse)

    def drop_the_Value(self,target,inverse=1):
        self._4ods_dropRelationship('the_Value','value_of','drop',target,inverse)

    def form_type(self,target,inverse = 1):
        from Ft.Ods.MetaData import Type
        self._4ods_formRelationship('type',Type.Type,'constants','add',target,inverse)

    def drop_type(self,target,inverse=1):
        self._4ods_dropRelationship('type','constants','remove',target,inverse)

    def form_referenced_by(self,target,inverse = 1):
        self.add_referenced_by(target,invers)

    def add_referenced_by(self,target,inverse=1):
        from Ft.Ods.MetaData import ConstOperand
        self._4ods_addRelationship('referenced_by',ConstOperand.ConstOperand,'references','form',target,inverse)

    def drop_referenced_by(self,target,inverse=1):
        self.remove_referenced_by(target,invers)

    def remove_referenced_by(self,target,inverse=1):
        self._4ods_removeRelationship('referenced_by','references','drop',target,inverse)

    def form_enumeration(self,target,inverse = 1):
        self._4ods_formRelationship('enumeration',Enumeration.Enumeration,'elements','add',target,inverse)

    def drop_enumeration(self,target,inverse=1):
        self._4ods_dropRelationship('enumeration','elements','remove',target,inverse)



    def _4ods_getOdl(self):
        st = 'const %s %s = %s;' % (self.type._4ods_getOdl(),self.name,self.the_Value._4ods_getOdl())
        return st

    readOnlyAttributes = MetaObject.MetaObject.readOnlyAttributes + ('the_Value',
                                                                     'type',
                                                                     'referenced_by',
                                                                     'enumeration'
                                                                     )
                                                                     
    readWriteAttributes = MetaObject.MetaObject.readWriteAttributes
    relationships = MetaObject.MetaObject.relationships + ('the_Value',
                                                           'type',
                                                           'referenced_by',
                                                           'enumeration'
                                                           )



    tupleNames = MetaObject.MetaObject.tupleNames  + ('the_Value',
                                                           'type',
                                                           'referenced_by',
                                                           'enumeration'
                                                           )
    tupleTypes = MetaObject.MetaObject.tupleTypes + (Constants.Types.OBJECT,
                                                     Constants.Types.OBJECT,
                                                     Constants.Types.OBJECT_SET,
                                                     Constants.Types.OBJECT
                                                     )
    
        
    localExtents = ()
    _collections = MetaObject.MetaObject._collections + ('referenced_by',)
    collectionMap = MetaObject.MetaObject.collectionMap.copy()
    collectionMap.update({'referenced_by':LiteralSetOfRepositoryObjects.LiteralSetOfRepositoryObjects}
                         )


