########################################################################
#
# File Name: 	       Enumeration.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/Enumeration.py.html
#
"""
Implements the Enumeration meta-data interface.
WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 1999 FourThought LLC, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

from Ft.Ods.MetaData import ScopedType
from Ft.Ods.Collections import LiteralListOfRepositoryObjects
from Ft.Ods.MetaData import RepositoryObject

from Ft.Ods.MetaData import Constant

from Ft.Ods.StorageManager.Adapters import Constants

class Enumeration(ScopedType.ScopedType):
    def __init__(self):
        ScopedType.ScopedType.__init__(self,RepositoryObject.MetaKind.mk_enumeration)


    #These three interfaces are for any scopedtype
    #We must support them
    def _4ods_getBoundChildren(self):
        return self.elements._4ods_getContents()
    def _4ods_boundChild(self,obj):
        self.add_elements(obj)
    def _4ods_unoundChild(self,obj):
        self.remove_elements(obj)


    def form_elements(self,target,inverse=1):
        self.add_elements(target,inverse)

    def add_elements(self,target,inverse=1):
        self._4ods_addRelationship('elements',Constant.Constant,'enumeration','form',target,inverse)

    def drop_elements(self,target,inverse = 1):
        self.remove_elements(target,inverse)
    def remove_elements(self,target,inverse = 1):
        self._4ods_removeRelationship('elements','enumerations','drop',target,inverse)

    def _4ods_getOdl(self):
        st = 'enum %s { ' % self.name
        for e in self.elements._4ods_getContents():
            st = st + ' %s,' % e.name
        st = st[:-1] + '};'
        return st

    def _4ods_getOdmgType(self):
        return Constants.Types.ENUMERATION

        
    readOnlyAttributes = ScopedType.ScopedType.readOnlyAttributes + ('elements',)
    readWriteAttributes = ScopedType.ScopedType.readWriteAttributes 
    relationships = ScopedType.ScopedType.relationships + ('elements',)

    tupleNames =  ScopedType.ScopedType.tupleNames + ('elements',
                                                    )
    tupleTypes =  ScopedType.ScopedType.tupleTypes + (Constants.Types.OBJECT_LIST,
                                                    )
        

    _collections = ScopedType.ScopedType._collections + ('elements',)
    collectionMap = ScopedType.ScopedType.collectionMap.copy()
    collectionMap.update({'elements':LiteralListOfRepositoryObjects.LiteralListOfRepositoryObjects
                          }
                         )
    localExtents = ()
