########################################################################
#
# File Name: 	        MetaObject.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/MetaObject.py.html
#
"""
Implements the MetaObject meta-data interface.
WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 1999 FourThought LLC, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

ftods_attributes = ['name', 'comment']

from Ft.Ods.MetaData import RepositoryObject
from Ft.Ods.StorageManager.Adapters import Constants

class MetaObject(RepositoryObject.RepositoryObject):
    def __init__(self,meta_kind):
        RepositoryObject.RepositoryObject.__init__(self,meta_kind)
        self.__dict__['name'] = ''
        self.__dict__['comment'] = ''

    def absolute_name(self):
        p = self.parent()
        if p:
            pas = p.absolute_name()
            if pas:
                pas = pas + '::'
            return pas + self.name
        return self.name

    def parent(self):
        if self.definedIn == None:
            return None
        if self.definedIn.meta_kind == RepositoryObject.MetaKind.mk_repository:
            
            return None
        return self.definedIn

    #relationship definedIn
    #FIXME: Check that passed-in parameters  the current relationship member
    def form_definedIn(self, definedIn, inverse=1):
        from Ft.Ods.MetaData import DefiningScope

        self._4ods_formRelationship('definedIn',DefiningScope.DefiningScope,'defines','add',definedIn,inverse)
        
    def drop_definedIn(self, definedIn, inverse=1):
        self._4ods_dropRelationship('definedIn','defines','remove',definedIn,inverse)



    readOnlyAttributes = RepositoryObject.RepositoryObject.readOnlyAttributes + ('definedIn',)
    readWriteAttributes = RepositoryObject.RepositoryObject.readWriteAttributes + ('name','comment')
    relationships = RepositoryObject.RepositoryObject.relationships + ('definedIn',)


    tupleNames = RepositoryObject.RepositoryObject.tupleNames + ('name',
                                                                 'comment',
                                                                 'definedIn'
                                                                 )

    tupleTypes = RepositoryObject.RepositoryObject.tupleTypes + (Constants.Types.STRING,
                                                                 Constants.Types.STRING,
                                                                 Constants.Types.OBJECT
                                                                 )

    _extents = RepositoryObject.RepositoryObject._extents
    localExtents = ()
    _collections = RepositoryObject.RepositoryObject._collections

    collectionMap = RepositoryObject.RepositoryObject.collectionMap.copy()
    
