########################################################################
#
# File Name: 	        Module.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/Module.py.html
#
"""
Implements the Module meta-data interface.
WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 1999 FourThought, Inc, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

from Ft.Ods.MetaData import MetaObject
from Ft.Ods.MetaData import DefiningScope
from Ft.Ods.MetaData import RepositoryObject

class Module(MetaObject.MetaObject, DefiningScope.DefiningScope):
    def __init__(self,meta_kind = RepositoryObject.MetaKind.mk_module):
        MetaObject.MetaObject.__init__(self, meta_kind)
        DefiningScope.DefiningScope.__init__(self)


    def add_module(self,name):
        m = Module()
        m.name = name
        self.bind(name,m)
        return m

    def add_interface(self,name, inherits):
        from Ft.Ods.MetaData import Interface
        i = Interface.Interface()
        i.name = name
        if inherits:
            for inherit in inherits._4ods_getContents():
                i.add_inherits(inherit)
        self.bind(name,i)
        return i

    def add_class(self,name,inherits,extender):
        from Ft.Ods.MetaData import Class
        c = Class.Class()
        c.name = name
        if inherits:
            for inherit in inherits._4ods_getContents():
                c.add_inherits(inherit)
        if extender:
            c.form_extender(extender)
        self.bind(name,c)
        return c

    def remove_module(self,mod):
        if not mod._4ods_checkRemove():
            raise DefiningScope.CannotRemove()
        self.remove_defines(mod)

    def remove_interface(self,i):
        if not i._4ods_checkRemove():
            raise DefiningScope.CannotRemove()
        self.remove_defines(i)

    def remove_class(self,c):
        if not c._4ods_checkRemove():
            raise DefiningScope.CannotRemove()
        self.remove_defines(c)


    def _4ods_getOdl(self):
        st = 'module %s {\n' % self.name
        for d in self.defines._4ods_getContents():
            if d.meta_kind == RepositoryObject.MetaKind.mk_interface:
                st = st + 'interface %s;\n' % d.absolute_name()
            if d.meta_kind == RepositoryObject.MetaKind.mk_class:
                st = st + 'class %s;\n' % d.absolute_name()
        for d in self.defines._4ods_getContents():
            st = st + '%s\n' % d._4ods_getOdl()
        st = st + '};\n'
        return st



    def _4ods_checkRemove(self):
        return 1


    readOnlyAttributes = MetaObject.MetaObject.readOnlyAttributes + DefiningScope.DefiningScope.readOnlyAttributes
    readWriteAttributes = MetaObject.MetaObject.readWriteAttributes + DefiningScope.DefiningScope.readWriteAttributes
    relationships = MetaObject.MetaObject.relationships + DefiningScope.DefiningScope.relationships



    tupleNames = MetaObject.MetaObject.tupleNames + DefiningScope.DefiningScope.tupleNames
    tupleTypes = MetaObject.MetaObject.tupleTypes + DefiningScope.DefiningScope.tupleTypes
    

    _collections = MetaObject.MetaObject._collections + DefiningScope.DefiningScope._collections
    collectionMap = MetaObject.MetaObject.collectionMap.copy()
    collectionMap.update(DefiningScope.DefiningScope.collectionMap)
    
        
    localExtents = ()
