########################################################################
#
# File Name: 	       Operand.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/Operand.py.html
#
"""
Implements the Operand meta-data interface.
WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 1999 FourThought LLC, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

from Ft.Ods.MetaData import RepositoryObject
from Ft.Ods.StorageManager.Adapters import Constants

from Ft.Ods.Collections import LiteralSetOfRepositoryObjects

class Operand(RepositoryObject.RepositoryObject):
    def __init__(self,meta_kind):
        RepositoryObject.RepositoryObject.__init__(self,meta_kind)

    def value(self):
        raise "Must be overridden in %s" % str(self)

    def form_operand_in(self,target,inverse=1):
        from Ft.Ods.MetaData import Expression
        self._4ods_formRelationship('operand_in',Expression.Expression,'the_operands','add',target,inverse)

    def drop_operand_in(self,target,inverse=1):
        self._4ods_dropRelationship('operand_in','the_operands','remove',target,inverse)

    def form_value_of(self,target,inverse = 1):
        from Ft.Ods.MetaData import Constant
        self._4ods_formRelationship('value_of',Constant.Constant,'the_value','form',target,inverse)

    def drop_value_of(self,target,inverse=1):
        self._4ods_dropRelationship('value_of','the_value','drop',target,inverse)

    def form_size_of(self,target,inverse = 1):
        from Ft.Ods.MetaData import Collection
        self._4ods_formRelationship('size_of',Collection.Collection,'max_size','form',target,inverse)

    def drop_size_of(self,target,inverse=1):
        self._4ods_dropRelationship('size_of','max_size','drop',target,inverse)

    def form_case_in(self,target,inverse = 1):
        from Ft.Ods.MetaData import UnionCase
        self._4ods_formRelationship('case_in',UnionCase.UnionCase,'case_labels','add',target,inverse)

    def drop_case_in(self,target,inverse=1):
        self._4ods_dropRelationship('case_in','case_labels','remove',target,inverse)



    readOnlyAttributes = RepositoryObject.RepositoryObject.readOnlyAttributes + ('operand_in',
                                                                     'value_of',
                                                                     'size_of',
                                                                     'case_in'
                                                                     )
                                                                     
    readWriteAttributes = RepositoryObject.RepositoryObject.readWriteAttributes
    relationships = RepositoryObject.RepositoryObject.relationships + ('operand_in',
                                                           'value_of',
                                                           'size_of',
                                                           'case_in'
                                                           )



    tupleNames = RepositoryObject.RepositoryObject.tupleNames  + ('operand_in',
                                                           'value_of',
                                                           'size_of',
                                                           'case_in'
                                                           )
    tupleTypes = RepositoryObject.RepositoryObject.tupleTypes + (Constants.Types.OBJECT,
                                                     Constants.Types.OBJECT,
                                                     Constants.Types.OBJECT_SET,
                                                     Constants.Types.OBJECT
                                                     )
    
        


    _collections = RepositoryObject.RepositoryObject._collections
    collectionMap = RepositoryObject.RepositoryObject.collectionMap
    localExtents = ()
