########################################################################
#
# File Name: 	       Operation.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/Operation.py.html
#
"""
Implements the Operation meta-data interface.
WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 1999 FourThought LLC, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""


from Ft.Ods.MetaData import MetaObject

from Ft.Ods.MetaData import Parameter
from Ft.Ods.MetaData import Exception
from Ft.Ods.MetaData import RepositoryObject
from Ft.Ods.StorageManager.Adapters import Constants
from Ft.Ods.Collections import LiteralListOfRepositoryObjects

class Operation(MetaObject.MetaObject):

    def __init__(self):
        MetaObject.MetaObject.__init__(self,RepositoryObject.MetaKind.mk_operation)


    def form_signature(self,target,inverse = 1):
        self.add_signature(target,inverse)

    def add_signature(self,target,inverse = 1):
        self._4ods_addRelationship('signature',Parameter.Parameter,'operation','form',target,inverse)

    def drop_signature(self,target,inverse = 1):
        self.remove_signature(target,inverse)

    def remove_signature(self,target,inverse = 1):
        self._4ods_removeRelationship('signature','operation','drop',target,inverse)

    def form_result(self,target,inverse = 1):
        from Ft.Ods.MetaData import Type
        self._4ods_formRelationship('result',Type.Type,'operations','add',target,inverse)
    def drop_result(self,target,inverse = 1):
        self._4ods_dropRelationship('result','operations','remove',target,inverse)

    
    def form_exceptions(self,target,inverse = 1):
        self.add_exceptions(target,inverse)

    def add_exceptions(self,target,inverse = 1):
        self._4ods_addRelationship('exceptions',Exception.Exception,'operations','form',target,inverse)

    def drop_exceptions(self,target,inverse = 1):
        self.remove_exceptions(target,inverse)

    def remove_exceptions(self,target,inverse = 1):
        self._4ods_removeRelationship('exceptions','operations','drop',target,inverse)

    readOnlyAttributes = MetaObject.MetaObject.readOnlyAttributes + ('signature',
                                                                     'result',
                                                                     'exceptions'
                                                                     )
    readWriteAttributes = MetaObject.MetaObject.readWriteAttributes
    relationships = MetaObject.MetaObject.relationships + ('signature',
                                                           'result',
                                                           'exceptions'
                                                           )



    tupleNames = MetaObject.MetaObject.tupleNames + ('signature',
                                                     'result',
                                                     'exceptions'
                                                     )
    tupleTypes = MetaObject.MetaObject.tupleTypes + (Constants.Types.OBJECT_LIST,
                                                     Constants.Types.OBJECT,
                                                     Constants.Types.OBJECT_LIST
                                                     )
    
    _collections = MetaObject.MetaObject._collections + ('signature','exceptions')
    collectionMap = MetaObject.MetaObject.collectionMap.copy()
    collectionMap.update({'signature':LiteralListOfRepositoryObjects.LiteralListOfRepositoryObjects,
                          'exceptions':LiteralListOfRepositoryObjects.LiteralListOfRepositoryObjects
                          }
                         )

    localExtents = ()
