########################################################################
#
# File Name: 	        Parameter.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/Parameter.py.html
#
"""
Implements the Parameter meta-data interface.
WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 1999 FourThought LLC, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""


from Ft.Ods.MetaData import Specifier
from Ft.Ods.MetaData import RepositoryObject

from Ft.Ods.StorageManager.Adapters import Constants
from Ft.Ods.Collections import LiteralListOfRepositoryObjects

class Parameter(Specifier.Specifier):
    def __init__(self,parameter_mode):
        Specifier.Specifier.__init__(self,RepositoryObject.MetaKind.mk_union_case)
        self.__dict__['parameter_mode'] = parameter_mode
        self.__dict__['operation'] = None

    def form_operation(self,target,inverse = 1):
        from Ft.Ods.MetaData import Operation
        self._4ods_addRelationship('operation',Operation.Operation,'signature','add',target,inverse)

    def drop_operation(self,target,inverse = 1):
        self._4ods_removeRelationship('operation','signature','form',target,inverse)


    readOnlyAttributes = Specifier.Specifier.readOnlyAttributes + ('operation',)
    readWriteAttributes = Specifier.Specifier.readWriteAttributes + ('parameter_mode',)
    relationships = Specifier.Specifier.relationships + ('operation',)

    tupleNames =  Specifier.Specifier.tupleNames + ('parameter_mode','operation'
                                                                  )
    tupleTypes =  Specifier.Specifier.tupleTypes + (Constants.Types.ENUMERATION,
                                                    Constants.Types.OBJECT
                                                    )

    _collections = Specifier.Specifier._collections
    collectionMap = Specifier.Specifier.collectionMap
    localExtents = ()

class DirectionEnumMember: pass

class DirectionEnum:
   def __init__(self):
        self.elements = []

   def lookup(self,st):
       for e in self.elements:
           if e._n == st: return e
       return ''
        
   class mode_in(DirectionEnumMember):
       _n = 'mode_in'
       _v = 0
   class mode_out(DirectionEnumMember):
       _n = 'mode_out'
       _v = 1
   class mode_inout(DirectionEnumMember):
       _n = 'mode_inout'
       _v = 2

Direction = DirectionEnum()
Direction.elements.append(DirectionEnum.mode_in)
Direction.elements.append(DirectionEnum.mode_out)
Direction.elements.append(DirectionEnum.mode_inout)



