########################################################################
#
# File Name: 	        PrimitiveType.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/PrimitiveType.py.html
#
"""
Implements the Type meta-data interface.
WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 1999 FourThought LLC, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""


from Ft.Ods.MetaData import Type
from Ft.Ods.MetaData import RepositoryObject

from Ft.Ods.StorageManager.Adapters import Constants



g_nameToOdlMap = {'pk_boolean' : 'boolean',
                  'pk_char' : 'char',
                  'pk_string' : 'string',
                  'pk_short' : 'short',
                  'pk_ushort' : 'unsigned short',
                  'pk_date' : 'date',
                  'pk_time' : 'time',
                  'pk_timestamp' : 'timestamp',
                  'pk_long' : 'long',
                  'pk_ulong' : 'unsigned long',
                  'pk_long_long' : 'long long',
                  'pk_float' : 'float',
                  'pk_double' : 'double',
                  'pk_octet' : 'octet',
                  'pk_interval' : 'interval',
                  'pk_void' : 'void',
                  'pk_file' : 'file'
                  }
g_nameToOdlType = {
    0:Constants.Types.BOOLEAN,
    2 : Constants.Types.STRING,
    3:Constants.Types.SIGNED_SHORT,
    4:Constants.Types.UNSIGNED_SHORT,
    8:Constants.Types.SIGNED_LONG,
    9:Constants.Types.UNSIGNED_LONG,
    10:Constants.Types.SIGNED_LONG_LONG,
    11: Constants.Types.FLOAT, 
    12:Constants.Types.DOUBLE,
    16:Constants.Types.FILE
    }

class PrimitiveType(Type.Type):

    def __init__(self,pk=None):
        Type.Type.__init__(self,RepositoryObject.MetaKind.mk_primitive_type)
        self.__dict__['primitive_kind'] = pk


    def absolute_name(self):
        return g_nameToOdlMap[self.name]
        

    def _4ods_getPythonClass(self,name):
        if name == 'primitive_kind':
            return PrimitiveKind
        return Type.Type._4ods_getPythonClass(self,name)

    def _4ods_getOdmgType(self):
        return g_nameToOdlType[self.primitive_kind._v]

    
    def _4ods_getOdl(self):
        return g_nameToOdlMap[self.name]

    readOnlyAttributes = Type.Type.readOnlyAttributes + ('primitive_kind',)
    readWriteAttributes = Type.Type.readWriteAttributes + ()
    relationships = Type.Type.relationships + ()

    tupleNames = Type.Type.tupleNames + ('primitive_kind',)
    tupleTypes = Type.Type.tupleTypes + (Constants.Types.ENUMERATION,)

    _collections = Type.Type._collections
    collectionMap = Type.Type.collectionMap
    localExtents = ()


class PrimitiveKindEnumMember: pass

class PrimitiveKindEnum:

    def __init__(self):
        self.elements = []

    def lookup(self,st):
        for e in self.elements:
            if e._n == st: return e
        return ''
    class pk_boolean(PrimitiveKindEnumMember):
        _n = 'pk_boolean'
        _v = 0
    class pk_char(PrimitiveKindEnumMember):
        _n = 'pk_char'
        _v = 1
    class pk_string(PrimitiveKindEnumMember):
        _n = 'pk_string'
        _v = 2
    class pk_short(PrimitiveKindEnumMember):
        _n = 'pk_short'
        _v = 3
    class pk_ushort(PrimitiveKindEnumMember):
        _n = 'pk_ushort'
        _v = 4
    class pk_date(PrimitiveKindEnumMember):
        _n = 'pk_date'
        _v = 5
    class pk_time(PrimitiveKindEnumMember):
        _n = 'pk_time'
        _v = 6
    class pk_timestamp(PrimitiveKindEnumMember):
        _n = 'pk_timestamp'
        _v = 7
    class pk_long(PrimitiveKindEnumMember):
        _n = 'pk_long'
        _v = 8
    class pk_ulong(PrimitiveKindEnumMember):
        _n = 'pk_ulong'
        _v = 9
    class pk_long_long(PrimitiveKindEnumMember):
        _n = 'pk_long_long'
        _v = 10
    class pk_float(PrimitiveKindEnumMember):
        _n = 'pk_float'
        _v = 11
    class pk_double(PrimitiveKindEnumMember):
        _n = 'pk_double'
        _v = 12
    class pk_octet(PrimitiveKindEnumMember):
        _n = 'pk_octet'
        _v = 13
    class pk_interval(PrimitiveKindEnumMember):
        _n = 'pk_interval'
        _v = 14
    class pk_void(PrimitiveKindEnumMember):
        _n = 'pk_void'
        _v = 15
    class pk_file(PrimitiveKindEnumMember):
        _n = 'pk_file'
        _v = 16
    
        
PrimitiveKind = PrimitiveKindEnum()
PrimitiveKind.elements.append(PrimitiveKind.pk_boolean)
PrimitiveKind.elements.append(PrimitiveKind.pk_char)
PrimitiveKind.elements.append(PrimitiveKind.pk_string)
PrimitiveKind.elements.append(PrimitiveKind.pk_short)
PrimitiveKind.elements.append(PrimitiveKind.pk_ushort)
PrimitiveKind.elements.append(PrimitiveKind.pk_date)
PrimitiveKind.elements.append(PrimitiveKind.pk_time)
PrimitiveKind.elements.append(PrimitiveKind.pk_timestamp)
PrimitiveKind.elements.append(PrimitiveKind.pk_long)
PrimitiveKind.elements.append(PrimitiveKind.pk_ulong)
PrimitiveKind.elements.append(PrimitiveKind.pk_long_long)
PrimitiveKind.elements.append(PrimitiveKind.pk_float)
PrimitiveKind.elements.append(PrimitiveKind.pk_double)
PrimitiveKind.elements.append(PrimitiveKind.pk_octet)
PrimitiveKind.elements.append(PrimitiveKind.pk_interval)
PrimitiveKind.elements.append(PrimitiveKind.pk_void)
PrimitiveKind.elements.append(PrimitiveKind.pk_file)

