########################################################################
#
# File Name: 	       Relationship.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/Relationship.py.html
#
"""
Implements the Relationship meta-data interface.
WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 1999 FourThought LLC, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

from Ft.Ods.MetaData import RepositoryObject
from Ft.Ods.MetaData import Property

from Ft.Ods.StorageManager.Adapters import Constants


class Relationship(Property.Property):
    def __init__(self):
        Property.Property.__init__(self, RepositoryObject.MetaKind.mk_relationship)
        



    def get_cardinality(self):
        ourCard = self._4ods_getCardinality()
        thierCard = self.__dict__['traversal']._4ods_getCardinality()
        return g_cardinalityMap[ourCard][thierCard]

    def form_traversal(self,trav,inverse = 1):
        self._4ods_formRelationship('traversal',Relationship,'traversal','form',trav,inverse)

    def drop_traversal(self,trav,inverse=1):
        self._4ods_dropRelationship('traversal','traversal','drop',trav,inverse)
    
    def _4ods_getOdl(self):
        inv = self.traversal.absolute_name()
        if self.type.meta_kind == RepositoryObject.MetaKind.mk_collection:
            st = 'relationship %s <%s> inverse %s;' % (self.type._4ods_getOdl(),self.name,inv)
        else:
            st = 'relationship %s %s inverse %s;' % (self.type.absolute_name(),self.name,inv)
        return st

    def _4ods_getCardinality(self):
        from Ft.Ods.MetaData import Collection
        if isinstance(self.type,Collection.Collection):
            return 'N'
        return '1'

    readOnlyAttributes = Property.Property.readOnlyAttributes + ('traversal',)
    readWriteAttributes = Property.Property.readWriteAttributes
    relationships = Property.Property.relationships + ('traversal',)

    tupleNames = Property.Property.tupleNames + ('traversal',
                                                 )

    tupleTypes = Property.Property.tupleTypes + (Constants.Types.OBJECT,
                                                 )


    _collections = Property.Property._collections
    collectionMap = Property.Property.collectionMap
    localExtents = ()

class CardinalityEnumMember:
    pass

class CardinalityEnum:

    def __init__(self):
        self.elements = []

    def lookup(self,st):
        rt = filter(lambda x,t=st: x._n == t,self.elements)
        if len(rt):
            return rt[0]
        return None

    class c1_1(CardinalityEnumMember):
        _n = 'c1_1'
        _v = 0
    class c1_N(CardinalityEnumMember):
        _n = 'c1_N'
        _v = 1
    class cN_1(CardinalityEnumMember):
        _n = 'cN_1'
        _v = 2
    class cN_N(CardinalityEnumMember):
        _n = 'cN_N'
        _v = 3

Cardinality = CardinalityEnum()
Cardinality.elements.append(Cardinality.c1_1)
Cardinality.elements.append(Cardinality.c1_N)
Cardinality.elements.append(Cardinality.cN_1)
Cardinality.elements.append(Cardinality.cN_N)


g_cardinalityMap = {'1': {'1':Cardinality.c1_1,
                          'N':Cardinality.c1_N
                          },
                    'N': {'1':Cardinality.cN_1,
                          'N':Cardinality.cN_N
                          }
                    }
