########################################################################
#
# File Name: 	        RepositoryObject.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/RepositoryObject.py.html
#
"""
Implements the Repository meta-data interface.
WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 2000 FourThought Inc, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

from Ft.Ods.StorageManager.Adapters import Constants

from Ft.Ods import Database
from Ft.Ods import PersistentObject

from Ft.Ods.StorageManager.Adapters import Constants


class RepositoryObject(PersistentObject.PersistentObject):
    def __init__(self,meta_kind):
        PersistentObject.PersistentObject.__init__(self)

        self.__dict__['_repoId'] = 0
        self.__dict__['meta_kind'] = meta_kind

    def parent(self):
        return None

    def delete(self):
        PersistentObject.PersistentObject.delete(self)

    def absolute_name(self):
        if hasattr(self,'name'):
            return self.name
        return ''

    def _4ods_setRepositoryId(self,rid):
        self.__dict__['_repoId'] = rid
    def _4ods_getRepositoryId(self):
        return self.__dict__['_repoId']
    def _4ods_getId(self):
        return self.__dict__['_repoId']
    def _4ods_getPersistentType(self):
        return Constants.ObjectTypes.REPO


    #We need this for enums
    def _4ods_getPythonClass(self,name):
        if name == 'meta_kind':
            return MetaKind
        raise "Unknown class based attribute %s" % name

    readWriteAttributes = ()
    readOnlyAttributes = ('_repoId','meta_kind')
    relationships = ()

    tupleNames =  ('_repoId','meta_kind')
    tupleTypes = (Constants.Types.UNSIGNED_SHORT,Constants.Types.ENUMERATION)

    _extents = ()
    localExtents = ()

    _collections = ()
    collectionMap = {}


class MetaKindEnumMember:
    pass

class MetaKindEnum:

    def __init__(self):
        self.elements = []

    def lookup(self,st):
        for e in self.elements:
            if e._n == st: return e
        return ''
    class mk_attribute(MetaKindEnumMember):
        _n = 'mk_boolean'
        _v = 0
    class mk_class(MetaKindEnumMember):
        _n = 'mk_class'
        _v = 1
    class mk_collection(MetaKindEnumMember):
        _n = 'mk_collection'
        _v = 2
    class mk_constant(MetaKindEnumMember):
        _n = 'mk_constant'
        _v = 3
    class mk_const_operand(MetaKindEnumMember):
        _n = 'mk_const_operand'
        _v = 4
    class mk_enumeration(MetaKindEnumMember):
        _n = 'mk_enumeration'
        _v = 5
    class mk_exception(MetaKindEnumMember):
        _n = 'mk_exception'
        _v = 6
    class mk_expression(MetaKindEnumMember):
        _n = 'mk_expression'
        _v = 7
    class mk_interface(MetaKindEnumMember):
        _n = 'mk_interface'
        _v = 8
    class mk_literal(MetaKindEnumMember):
        _n = 'mk_literal'
        _v = 9
    class mk_member(MetaKindEnumMember):
        _n = 'mk_member'
        _v = 10
    class mk_module(MetaKindEnumMember):
        _n = 'mk_module'
        _v = 11
    class mk_operation(MetaKindEnumMember):
        _n = 'mk_operation'
        _v = 12
    class mk_parameter(MetaKindEnumMember):
        _n = 'mk_parameter'
        _v = 13
    class mk_primitive_type(MetaKindEnumMember):
        _n = 'mk_primitive_type'
        _v = 14
    class mk_relationship(MetaKindEnumMember):
        _n = 'mk_relationship'
        _v = 15
    class mk_repository(MetaKindEnumMember):
        _n = 'mk_repository'
        _v = 16
    class mk_structure(MetaKindEnumMember):
        _n = 'mk_strucutre'
        _v = 17
    class mk_type_definition(MetaKindEnumMember):
        _n = 'mk_type_definition'
        _v = 18
    class mk_union(MetaKindEnumMember):
        _n = 'mk_union'
        _v = 0
    class mk_union_case(MetaKindEnumMember):
        _n = 'mk_union_case'
        _v = 0



MetaKind = MetaKindEnum()
MetaKind.elements.append(MetaKind.mk_attribute)
MetaKind.elements.append(MetaKind.mk_class)
MetaKind.elements.append(MetaKind.mk_collection)
MetaKind.elements.append(MetaKind.mk_constant)
MetaKind.elements.append(MetaKind.mk_const_operand)
MetaKind.elements.append(MetaKind.mk_enumeration)
MetaKind.elements.append(MetaKind.mk_exception)
MetaKind.elements.append(MetaKind.mk_expression)
MetaKind.elements.append(MetaKind.mk_interface)
MetaKind.elements.append(MetaKind.mk_literal)
MetaKind.elements.append(MetaKind.mk_member)
MetaKind.elements.append(MetaKind.mk_module)
MetaKind.elements.append(MetaKind.mk_operation)
MetaKind.elements.append(MetaKind.mk_parameter)
MetaKind.elements.append(MetaKind.mk_primitive_type)
MetaKind.elements.append(MetaKind.mk_relationship)
MetaKind.elements.append(MetaKind.mk_repository)
MetaKind.elements.append(MetaKind.mk_structure)
MetaKind.elements.append(MetaKind.mk_type_definition)
MetaKind.elements.append(MetaKind.mk_union)
MetaKind.elements.append(MetaKind.mk_union_case)


