########################################################################
#
# File Name: 	        Structure.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/Structure.py.html
#
"""
Implements the Structure meta-data interface.
WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 1999 FourThought LLC, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""


from Ft.Ods.MetaData import ScopedType
from Ft.Ods.StorageManager.Adapters import Constants
from Ft.Ods.MetaData import Member
from Ft.Ods.Collections import LiteralListOfRepositoryObjects
from Ft.Ods.MetaData import RepositoryObject

class Structure(ScopedType.ScopedType):
    def __init__(self):
        ScopedType.ScopedType.__init__(self,RepositoryObject.MetaKind.mk_structure)



    def form_fields(self,target,inverse = 1):
        self.add_fields(target,inverse)

    def add_fields(self,target,inverse = 1):
        self._4ods_addRelationship('fields',Member.Member,'structure_type','form',target,inverse)
    def drop_fields(self,target,inverse = 1):
        self.remove_fields(target,inverse = 1)

    def remove_fields(self,target,inverse = 1):
        self._4ods_removeRelationship('fields','structure_type','drop',target,inverse)

    def form_exception_result(self,target,inverse = 1):
        from ft.Ods.MetaData import Exception
        self._4ods_formRelationship('exception_result',Exception.Exception,'result','form',target,inverse)
    def drop_exception_result(self,target,inverse = 1):
        self._4ods_dropRelationship('exception_result','result','drop',target,inverse)


    def _4ods_getOdl(self):
        st = 'struct %s {\n' % self.name
        for m in self.fields._4ods_getContents():
            st = st + m._ftods_getOdl() + ';\n'
        st = st + '};\n'
        return st

    def _4ods_getOdmgType(self):
        return Constants.Types.STRUCTURE


    def _4ods_getBoundChildren(self):
        return self.fields._4ods_getContents()
    def _4ods_boundChild(self,obj):
        self.add_fields(obj)
    def _4ods_unoundChild(self,obj):
        self.remove_fields(obj)




        
    readOnlyAttributes = ScopedType.ScopedType.readOnlyAttributes + ('fields','exception_result')
    readWriteAttributes = ScopedType.ScopedType.readWriteAttributes 
    relationships = ScopedType.ScopedType.relationships + ('fields','exception_result')

    tupleNames =  ScopedType.ScopedType.tupleNames + ('fields','exception_result'
                                                    )
    tupleTypes =  ScopedType.ScopedType.tupleTypes + (Constants.Types.OBJECT_LIST,
                                                      Constants.Types.OBJECT
                                                    )

    _collections = ScopedType.ScopedType._collections + ('fields',)
    collectionMap = ScopedType.ScopedType.collectionMap.copy()
    collectionMap.update({'fields':LiteralListOfRepositoryObjects.LiteralListOfRepositoryObjects
                          }
                         )


    localExtents = ()
