########################################################################
#
# File Name: 	        Type.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/Type.py.html
#
"""
Implements the Type meta-data interface.
WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 1999 FourThought LLC, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

from Ft.Ods.MetaData import MetaObject
from Ft.Ods.Collections import LiteralSetOfRepositoryObjects

from Ft.Ods.StorageManager.Adapters import Constants

#ftods_attributes = ['name', 'comment']

class Type(MetaObject.MetaObject):
    def __init__(self, meta_kind):
        MetaObject.MetaObject.__init__(self,meta_kind)

    #relationship properties
    def form_collections(self,target,inverse=1):
        self.add_collections(self,target,inverse)

    def add_collections(self,target,inverse=1):
        from Ft.Ods.MetaData import Collection
        self._4ods_addRelationship('collections',Collection.Collection,'subType','form',target,inverse)

    def drop_collections(self,target,inverse=1):
        self.remove_collections(self,target,inverse=1)

    def remove_collections(self,target,inverse=1):
        self._4ods_removeRelationship('collections','subType','drop',target,inverse)


    def form_dictionaries(self,target,inverse=1):
        self.add_dictionaries(self,target,inverse)

    def add_dictionaries(self,target,inverse=1):
        from Ft.Ods.MetaData import Dictionary
        self._4ods_addRelationship('dictionaries',Dictionary.Dictionary,'key_type','form',target,inverse)

    def drop_dictionaries(self,target,inverse=1):
        self.remove_dictionaries(self,target,inverse=1)

    def remove_dictionaries(self,target,inverse=1):
        self._4ods_removeRelationship('dictionaries','key_type','drop',target,inverse)


    def form_specifiers(self,target,inverse=1):
        self.add_specifiers(self,target,inverse)

    def add_specifiers(self,target,inverse=1):
        from Ft.Ods.MetaData import Specifier
        self._4ods_addRelationship('specifiers',Specifier.Specifier,'type','form',target,inverse)

    def drop_specifiers(self,target,inverse=1):
        self.remove_specifiers(self,target,inverse=1)

    def remove_specifiers(self,target,inverse=1):
        self._4ods_removeRelationship('specifiers','type','drop',target,inverse)


    def form_unions(self,target,inverse=1):
        self.add_unions(self,target,inverse)

    def add_unions(self,target,inverse=1):
        from Ft.Ods.MetaData import Union
        self._4ods_addRelationship('unions',Union.Union,'switch_type','form',target,inverse)

    def drop_unions(self,target,inverse=1):
        self.remove_unions(self,target,inverse=1)

    def remove_unions(self,target,inverse=1):
        self._4ods_removeRelationship('unions','switch_type','drop',target,inverse)


    def form_operations(self,target,inverse=1):
        self.add_operations(self,target,inverse)

    def add_operations(self,target,inverse=1):
        from Ft.Ods.MetaData import Operation
        self._4ods_addRelationship('operations',Operation.Operation,'result','form',target,inverse)

    def drop_operations(self,target,inverse=1):
        self.remove_operations(self,target,inverse=1)

    def remove_operations(self,target,inverse=1):
        self._4ods_removeRelationship('operations','result','drop',target,inverse)


    def form_properties(self,target,inverse=1):
        self.add_properties(self,target,inverse)

    def add_properties(self,target,inverse=1):
        from Ft.Ods.MetaData import Property
        self._4ods_addRelationship('properties',Property.Property,'type','form',target,inverse)

    def drop_properties(self,target,inverse=1):
        self.remove_properties(self,target,inverse=1)

    def remove_properties(self,target,inverse=1):
        self._4ods_removeRelationship('properties','type','drop',target,inverse)


    def form_constants(self,target,inverse=1):
        self.add_constants(self,target,inverse)

    def add_constants(self,target,inverse=1):
        from Ft.Ods.MetaData import Constant
        self._4ods_addRelationship('constants',Constant.Constant,'type','form',target,inverse)

    def drop_constants(self,target,inverse=1):
        self.remove_constants(self,target,inverse=1)

    def remove_constants(self,target,inverse=1):
        self._4ods_removeRelationship('constants','type','drop',target,inverse)

    def form_type_defs(self,target,inverse=1):
        self.add_type_defs(self,target,inverse)

    def add_type_defs(self,target,inverse=1):
        from Ft.Ods.MetaData import TypeDefinition
        self._4ods_addRelationship('type_defs',TypeDefinition.TypeDefinition,'alias','form',target,inverse)

    def drop_type_defs(self,target,inverse=1):
        self.remove_type_defs(self,target,inverse=1)

    def remove_type_defs(self,target,inverse=1):
        self._4ods_removeRelationship('type_defs','alias','drop',target,inverse)


    def _4ods_getOdmgType(self):
        raise "Not Implemented on class %s" % self.__class__.__name__




    readOnlyAttributes = MetaObject.MetaObject.readOnlyAttributes +  ('collections',
                                                                      'dictionaries',
                                                                      'specifiers',
                                                                      'unions',
                                                                      'operations',
                                                                      'properties',
                                                                      'constants',
                                                                      'type_defs'
                                                                   )
    readWriteAttributes = MetaObject.MetaObject.readWriteAttributes
    relationships = MetaObject.MetaObject.relationships + ('collections',
                                                          'dictionaries',
                                                          'specifiers',
                                                          'unions',
                                                          'operations',
                                                          'properties',
                                                          'constants',
                                                          'type_defs'
                                                          )




    tupleNames = MetaObject.MetaObject.tupleNames  + ('collections',
                                                      'dictionaries',
                                                      'specifiers',
                                                      'unions',
                                                      'operations',
                                                      'properties',
                                                      'constants',
                                                      'type_defs'
                                                      )


    tupleTypes = MetaObject.MetaObject.tupleTypes + (Constants.Types.OBJECT_SET,
                                                     Constants.Types.OBJECT_SET,
                                                     Constants.Types.OBJECT_SET,
                                                     Constants.Types.OBJECT_SET,
                                                     Constants.Types.OBJECT_SET,
                                                     Constants.Types.OBJECT_SET,
                                                     Constants.Types.OBJECT_SET,
                                                     Constants.Types.OBJECT_SET
                                                     )

    _collections = MetaObject.MetaObject._collections + ('collections',
                                                      'dictionaries',
                                                      'specifiers',
                                                      'unions',
                                                      'operations',
                                                      'properties',
                                                      'constants',
                                                      'type_defs'
                                                      )

    _extents = MetaObject.MetaObject._extents + ('ftods_types',)
    localExtents = ('ftods_types',)

    collectionMap = MetaObject.MetaObject.collectionMap.copy()
    collectionMap.update({'collections':LiteralSetOfRepositoryObjects.LiteralSetOfRepositoryObjects,
                          'dictionaries':LiteralSetOfRepositoryObjects.LiteralSetOfRepositoryObjects,
                          'specifiers':LiteralSetOfRepositoryObjects.LiteralSetOfRepositoryObjects,
                          'unions':LiteralSetOfRepositoryObjects.LiteralSetOfRepositoryObjects,
                          'operations':LiteralSetOfRepositoryObjects.LiteralSetOfRepositoryObjects,
                          'properties':LiteralSetOfRepositoryObjects.LiteralSetOfRepositoryObjects,
                          'constants':LiteralSetOfRepositoryObjects.LiteralSetOfRepositoryObjects,
                          'type_defs':LiteralSetOfRepositoryObjects.LiteralSetOfRepositoryObjects
                          }
                         )





