########################################################################
#
# File Name: 	        UnionCase.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/UnionCase.py.html
#
"""
Implements the UnionCase meta-data interface.
WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 1999 FourThought LLC, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""


from Ft.Ods.MetaData import Specifier
from Ft.Ods.MetaData import RepositoryObject

from Ft.Ods.StorageManager.Adapters import Constants
from Ft.Ods.MetaData import Operand
from Ft.Ods.Collections import LiteralListOfRepositoryObjects

class UnionCase(Specifier.Specifier):
    def __init__(self):
        Specifier.Specifier.__init__(self,RepositoryObject.MetaKind.mk_union_case)

    def form_union_type(self,target,inverse = 1):
        from Ft.Ods.MetaData import Union
        self._4ods_formRelationship('union_type',Union.Union,'cases','add',target,inverse)
    def drop_union_type(self,target,inverse = 1):
        self._4ods_dropRelationship('union_type','cases','remove',target,inverse)

    def form_case_labels(self,target,inverse = 1):
        self.add_case_labels(target,inverse)

    def add_case_labels(self,target,inverse = 1):
        self._4ods_addRelationship('case_labels',Operand.Operand,'case_in','form',target,inverse)

    def drop_case_labels(self,target,inverse = 1):
        self.remove_case_labels(target,inverse)

    def remove_case_labels(self,target,inverse = 1):
        self._4ods_removeRelationship('case_labels','case_in','drop',target,inverse)


    readOnlyAttributes = Specifier.Specifier.readOnlyAttributes + ('union_type','case_labels')
    readWriteAttributes = Specifier.Specifier.readWriteAttributes
    relationships = Specifier.Specifier.relationships + ('union_type','case_labels')

    tupleNames =  Specifier.Specifier.tupleNames + ('union_type','case_labels'
                                                                  )
    tupleTypes =  Specifier.Specifier.tupleTypes + (Constants.Types.OBJECT,
                                                    Constants.Types.OBJECT_LIST
                                                    )


    _collections = Specifier.Specifier._collections + ('case_labels',)
    collectionMap = Specifier.Specifier.collectionMap.copy()
    collectionMap.update({'case_labels':LiteralListOfRepositoryObjects.LiteralListOfRepositoryObjects
                          }
                         )

    localExtents = ()
