#!/usr/bin/env python
############################################################################
#
# File Name: 		Object.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/Object.py.html
#


from Ft.Ods.StorageManager.Adapters import Constants

#Lock Types
class Lock_Type:
    def __init__(self, typeName):
	self.__typeName = typeName

    def conflictsWith(self, newMode):
        conflict = 1
        if self.__typeName == 'read' and newMode.__typeName == 'read':
            conflict = 0
        return conflict

    def __repr__(self):
	return self.__typeName

read = Lock_Type("read")
write = Lock_Type("write")
upgrade = Lock_Type("upgrade")

#Exception class
class LockNotGranted:
    pass


from Ft.Ods import PersistentObject

class Object(PersistentObject.PersistentObject):
    def __init__(self):
        PersistentObject.PersistentObject.__init__(self)
        self.__dict__['_oid'] = 0


    def lock(self, mode):
        raise "Not implemented"

    def try_lock(self, mode):
        raise "Not implemented"

    def same_as(self,other):
        return isinstance(other,Object) and other._4ods_getOid() == self._4ods_getOid()

    def copy(self,deep=0,alreadyCloned=None):
        initArgs = ()
        if hasattr(self,'__getinitargs__'):
            initArgs = self.__getinitargs__()


        ac = alreadyCloned or {}

        names = self.tupleNames
        relationships = self.relationships
        types = self.tupleTypes
        Types = Constants.Types
        lists = Constants.g_listTypes
        newSelf = apply(self.__class__,initArgs)

        ac[self] = newSelf

        #We need to walk all of the types and determine waht to do in each case
        for n,t in map(lambda x,y: (x,y),names,types):
            if t in [Types.OBJECT]:
                #If it is an attribute, we copy it
                #If it is a relationship, we only copy it if they specify deep
                if n in relationships:
                    if deep:
                        #Clone the relationship as well
                        curobj = getattr(self,n)
                        if not curobj:
                            continue
                        newobj = ac.get(curobj)
                        if newobj is None:
                            #Has not been done yet so do the form
                            newobj = curobj.copy(deep,alreadyCloned=ac)
                            ac[curobj] = newobj
                        if not getattr(newSelf,n):
                            func = getattr(newSelf,'form_%s'%n)
                            apply(func,(newobj,0))
                else:
                    curobj = getattr(self,n)
                    if not curobj:
                        continue
                    newobj = ac.get(curobj)
                    if newobj is None:
                        #Has not been done yet so do the form
                        newobj = curobj.copy(deep,alreadyCloned=ac)
                    setattr(newSelf,n,newobj)

            elif lists[t]:
                if n in relationships:
                    if deep:
                        curcoll = getattr(self,n)
                        newcoll = getattr(newSelf,n)
                        for curobj in curcoll._4ods_getContents():
                            newobj = ac.get(curobj)
                            if newobj is None:
                                #Has not been done yet so do the add
                                newobj = curobj.copy(deep=0)
                                ac[curobj] = newobj
                            if not newcoll.contains_element(newobj):
                                func = getattr(newSelf,'add_%s'%n)
                                apply(func,(newobj,0))                        
                else:
                    #Just copy the list
                    curobj = getattr(self,n)
                    if not curobj:
                        continue
                    newobj = curobj.copy(deep=1)
                    setattr(newSelf,n,newobj)
                    
            elif t in [Types.STRING,Types.UNSIGNED_SHORT,Types.BOOLEAN,Types.UNSIGNED_LONG,Types.SIGNED_SHORT,Types.DOUBLE,Types.SIGNED_LONG_LONG, Types.SIGNED_LONG, Types.FLOAT, Types.FILE,Types.ENUMERATION]:
                if n != '_oid':
                    setattr(newSelf,n,getattr(self,n))
                            
        return newSelf

    def _4ods_getOid(self):
	return self.__dict__['_oid']

    def _4ods_setOid(self,id):
	self.__dict__['_oid'] = id

    def _4ods_getId(self):
	return self.__dict__['_oid']    

    def _4ods_getPersistentType(self):
        return Constants.ObjectTypes.OBJECT

    def _4ods_getRepositoryObject(self):
        return self.__dict__['_db']._4ods_getPersistentObjectById(Constants.ObjectTypes.REPO,self.typeId)

    readOnlyAttributes = ('_oid',)
    readWriteAttributes = ()
    relationships = ()

    tupleNames = ('_oid',)
    tupleTypes = (Constants.Types.UNSIGNED_LONG,)
    tupleIds = (None,)

    _extents = ()
    localExtents = ()


    _collections = ()
    collectionMap = {}
