#!/usr/bin/python
########################################################################
#
# File Name:            FILE
#
# Documentation:        http://docs.4suite.com/4ODS/Parsers/GenerateClass.py.html
#

#Desc.
#WWW: http://4suite.com/4ODS        e-mail: support@4suite.com

#Copyright (c) 1999 FourThought LLC, USA.   All Rights Reserved.
#See  http://4suite.com/COPYRIGHT  for license and copyright information

import string

from Ft.Ods.Parsers.Odl import GenerateUtils
from Ft.Ods.Parsers.Odl import GenerateType

from Ft.Ods.MetaData import RepositoryObject
from Ft.Ods.MetaData import PrimitiveType

import os

def GenerateStubs(i,params):
    #Shift directories

    classDir = os.path.join(params.curDir, i.name)
    if not os.path.exists(classDir):
        os.mkdir(classDir)

    params.curDir = classDir

    #Write our file
    intName = i.name

    outFileName = os.path.join(params.curDir, '__init__.py')
    outFile = open(outFileName,'w')
    params.outFile = outFile
    #print the imports
    outFile.write("#Generated file.  Please subclass rather than editing.\n")


    bClasses = []
    for inh in i.inherits._4ods_getContents():
        aName = string.split(inh.absolute_name(),'::')
        if aName[0] == 'ODLMetaObjects':
            aName = aName[1:]
        name = string.join(aName,'.')
        bClasses.append('%s.%s_stub' % (name,inh.name))
        GenerateType.WriteImports(inh,params)
    if i.meta_kind._n == 'mk_class' and i.extender:
        aName = string.split(i.extender.absolute_name(),'::')
        if aName[0] == 'ODLMetaObjects':
            aName = aName[1:]
        name = string.join(aName,'.')
        bClasses.append('%s.%s_stub' % (name,i.extender.name))
        GenerateType.WriteImports(i.extender,params)
    if len(bClasses) == 0:
        bClasses.append('Object.Object')


    #Write our base class imports
    if len(bClasses) == 1:
        outFile.write("from Ft.Ods import Object\n")
    outFile.write("from Ft.Ods import Database\n")


    if i.meta_kind._n == 'mk_class':

        #Write a global factory
        outFile.write('class %sFactory:\n'%intName)
        outFile.write('    def new(self):\n')
        outFile.write('        return %s_stub()\n'%intName)
        outFile.write('g_%sFactory = %sFactory()\n'%(intName, intName))
        outFile.write('def new():\n')
        outFile.write('    return g_%sFactory.new()\n\n' % intName)



    #Write the class definition and the constructor
    #FIXME no support for property_lists

    params.outFile.write("\nclass %s_stub(" % i.name)
    for bc in bClasses:
        params.outFile.write("%s" % (bc))
        if bc != bClasses[-1]:
            params.outFile.write(",")

    params.outFile.write("):\n\n")

    #Write class initialization
    params.outFile.write("\tdef __init__(self):\n")
    for bc in bClasses:
        params.outFile.write("\t\t%s.__init__(self)\n"%(bc))

    #Write the init for attributes and relationships
    for d in i.defines._4ods_getContents():
        if d.meta_kind in [RepositoryObject.MetaKind.mk_attribute]:
            if d.type.meta_kind not in  [RepositoryObject.MetaKind.mk_class,RepositoryObject.MetaKind.mk_interface,RepositoryObject.MetaKind.mk_collection]:
                if d.type.meta_kind == RepositoryObject.MetaKind.mk_primitive_type and d.type.primitive_kind == PrimitiveType.PrimitiveKind.pk_file:
                    continue
                params.outFile.write("\t\tself.__dict__['%s'] = "% d.name)
                GenerateType.WriteInit(d.type,params)
                params.outFile.write("\n")
        if d.meta_kind in [RepositoryObject.MetaKind.mk_structure]:
            raise "Fix Me"


    for d in i.defines._4ods_getContents():
        if d.meta_kind in [RepositoryObject.MetaKind.mk_relationship]:
            #Write form and drop
            if d.traversal._4ods_getCardinality() == 'N':
                inverseMethod = 'add'
            else:
                inverseMethod = 'form'

            params.outFile.write("\tdef form_%s(self,target,inverse=1):\n" % d.name)
            aName = string.split(d.traversal.definedIn.absolute_name(),'::')
            if aName[0] == 'ODLMetaObjects':
                aName = aName[1:]
            cName = string.join(aName,'.') + '.' + aName[-1] + '_stub'
            

            if d._4ods_getCardinality() == '1':
                params.outFile.write('\t\t')
                GenerateType.WriteImports(d.type,params)
                params.outFile.write("\t\tself._4ods_formRelationship('%s',%s,'%s','%s',target,inverse)\n" % (d.name,cName,d.traversal.name,inverseMethod))

            else:
                params.outFile.write("\t\tself.add_%s(target,inverse)\n" % d.name)
            
                params.outFile.write("\tdef add_%s(self,target,inverse=1):\n" % d.name)
                params.outFile.write('\t\t')
                GenerateType.WriteImports(d.type.subtype,params)
                params.outFile.write("\t\tself._4ods_addRelationship('%s',%s,'%s','%s',target,inverse)\n" % (d.name,cName,d.traversal.name,inverseMethod))
                

            if d.traversal._4ods_getCardinality() == 'N':
                inverseMethod = 'remove'
            else:
                inverseMethod = 'drop'

            params.outFile.write("\tdef drop_%s(self,target,inverse=1):\n" % d.name)

            if d._4ods_getCardinality() == '1':
                params.outFile.write("\t\tself._4ods_dropRelationship('%s','%s','%s',target,inverse)\n" % (d.name,d.traversal.name,inverseMethod))

            else:
                params.outFile.write("\t\tself.remove_%s(target,inverse)\n" % d.name)
            
                params.outFile.write("\tdef remove_%s(self,target,inverse=1):\n" % d.name)

                params.outFile.write("\t\tself._4ods_removeRelationship('%s','%s','%s',target,inverse)\n" % (d.name,d.traversal.name,inverseMethod))

    ###Needed if we have enum children
    params,outFile.write("\tdef _4ods_getPythonClass(self,name):\n")
    for d in i.defines._4ods_getContents():
        if d.type.meta_kind == RepositoryObject.MetaKind.mk_enumeration:
            path = d.type.absolute_name()
            path = string.split(path,'::')
            if path[0] == 'ODLMetaObjects':
                path = path[1:]
            cPath = string.join(path,'.')
            params.outFile.write("\t\tif name == '%s': return %s.%s\n" % (d.name,cPath,path[-1]))

    for bc in bClasses:
        params.outFile.write("\t\trt = %s._4ods_getPythonClass(self,name)\n" % (bc))
        params.outFile.write("\t\tif rt:return rt\n" )
    params,outFile.write("\t\treturn None\n\n\n")
    


    ###Some Helpers
    params.outFile.write("\tdef _4ods_getType(self):\n")
    params.outFile.write("\t\treturn self.__dict__['__type']\n")


    #Write the tuple info
    readOnlyAttributes = []
    readWriteAttributes = []
    relationships = []
    tupleNames = []
    tupleTypes = []
    tupleIds = []

    collections = []
    collectionMap = []


    for d in i.defines._4ods_getContents():
        if d.meta_kind == RepositoryObject.MetaKind.mk_attribute:
            if d.is_read_only:
                readOnlyAttributes.append(d.name)
            else:
                readWriteAttributes.append(d.name)
            tupleNames.append(d.name)
            tupleTypes.append(d._4ods_getOdmgType())
            tupleIds.append(d._4ods_getRepositoryId())
            if d.type.meta_kind == RepositoryObject.MetaKind.mk_collection:
                collections.append((d.name,d.type))
        if d.meta_kind == RepositoryObject.MetaKind.mk_relationship:
            relationships.append(d.name)
            tupleNames.append(d.name)
            tupleTypes.append(d._4ods_getOdmgType())
            tupleIds.append(d._4ods_getRepositoryId())
            if d.type.meta_kind == RepositoryObject.MetaKind.mk_collection:
                collections.append((d.name,d.type))
    extents = []
    if i.meta_kind._n == 'mk_class':
        for ext in i.extents:
            extents.append(ext)

    params.outFile.write("\t_extents = ")
    for bc in bClasses:
        params.outFile.write("%s._extents" % (bc))
        params.outFile.write(" + ")
    params.outFile.write(str(tuple(extents)))
    params.outFile.write("\n")
    params.outFile.write("\tlocalExtents = ")
    params.outFile.write(str(tuple(extents)))
    params.outFile.write("\n")
    
    

    params.outFile.write("\treadOnlyAtributes = ")
    for bc in bClasses:
        params.outFile.write("%s.readOnlyAttributes" % (bc))
        params.outFile.write(" + ")
    params.outFile.write(str(tuple(readOnlyAttributes)))
    params.outFile.write("\n")

    params.outFile.write("\treadWriteAttributes = ")
    for bc in bClasses:
        params.outFile.write("%s.readWriteAttributes" % (bc))
        params.outFile.write(" + ")
    params.outFile.write(str(tuple(readWriteAttributes)))
    params.outFile.write("\n")

    params.outFile.write("\trelationships = ")
    for bc in bClasses:
        params.outFile.write("%s.relationships" % (bc))
        params.outFile.write(" + ")
    params.outFile.write(str(tuple(relationships)))
    params.outFile.write("\n")

    params.outFile.write("\ttupleNames = ")
    for bc in bClasses:
        params.outFile.write("%s.tupleNames" % (bc))
        params.outFile.write(" + ")
    params.outFile.write(str(tuple(tupleNames)))
    params.outFile.write("\n")

    params.outFile.write("\ttupleTypes = ")
    for bc in bClasses:
        params.outFile.write("%s.tupleTypes" % (bc))
        params.outFile.write(" + ")
    params.outFile.write(str(tuple(tupleTypes)))
    params.outFile.write("\n")

    params.outFile.write("\ttupleIds = ")
    for bc in bClasses:
        params.outFile.write("%s.tupleIds" % (bc))
        params.outFile.write(" + ")
    params.outFile.write(str(tuple(tupleIds)))
    params.outFile.write("\n")

    params.outFile.write("\t_collections = ")
    for bc in bClasses:
        params.outFile.write("%s._collections" % (bc))
        params.outFile.write(" + ")
    params.outFile.write(str(tuple(map(lambda x: x[0],collections))))
    params.outFile.write("\n")


    
    params.outFile.write("\tcollectionMap = %s.collectionMap.copy()\n" % bClasses[0])
    for bc in bClasses[1:]:
        params.outFile.write("\tcollectionMap.update(%s.collectionMap)\n" % bClasses[0])

    
    cNames = []
    st = "\tcollectionMap.update({"
    for c in collections:
        cName = GenerateType.CollectionToName(c[1])
        if not cName in cNames:
            cNames.append(cName)
        st = st + "'%s':%s.%s," % (c[0],cName,cName)
        
    st = st + "})\n"
        
    for cName in cNames:
        params.outFile.write("\tfrom Ft.Ods.Collections import %s\n" % (cName))

    params.outFile.write(st)
    


    params.outFile.write("\ttypeId = %i\n" % i._4ods_getRepositoryId())




    for e in i.defines._4ods_getContents():
        #Write any imports we need for any object attributes and object relationsips
        if e.meta_kind != RepositoryObject.MetaKind.mk_attribute and e.meta_kind != RepositoryObject.MetaKind.mk_relationship:
            GenerateType.WriteImports(e.type,params)
        elif e.meta_kind == RepositoryObject.MetaKind.mk_attribute and e.type.meta_kind == RepositoryObject.MetaKind.mk_enumeration:
            GenerateType.WriteImports(e.type,params)
            

    outFile.write('\n')
    outFile.close()

    
    #Last but not least, see if any of defines need to do thier own generation
    for e in i.defines._4ods_getContents():
        if e.meta_kind in [RepositoryObject.MetaKind.mk_constant
                           ]:
            GenerateUtils.ExecuteDefinition(e,'GenerateStubs',params)
