########################################################################
#
# File Name: 	    DbmHelper.py
#
# Documentation:	http://docs.4suite.or/4ODS/StorageManager/Adapters/DbmHelper.py.html
#
"""
The DBM Helper Functions back-end.
WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 1999 FourThought, Inc, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

import string

import Constants
import DbmMappings
from Ft.Lib import DbmDatabase

from Ft.Ods import FtodsBindingException
from Ft.Ods import FtodsObjectNotFound


class DbmHelper:

    def _init(self):
        self._importMetaData()
        self._loadPythonClasses()
        

    #Init Helpers
    def _importMetaData(self):
        if len(DbmMappings.g_metaKindToRepoClass):
            return

        from Ft.Ods.MetaData import Attribute
        DbmMappings.g_metaKindToRepoClass[0] = Attribute.Attribute

        from Ft.Ods.MetaData import Class
        DbmMappings.g_metaKindToRepoClass[1] = Class.Class

        from Ft.Ods.MetaData import Collection
        DbmMappings.g_metaKindToRepoClass[2] = Collection.Collection

        from Ft.Ods.MetaData import Constant
        DbmMappings.g_metaKindToRepoClass[3] = Constant.Constant

        from Ft.Ods.MetaData import ConstOperand
        DbmMappings.g_metaKindToRepoClass[4] = ConstOperand.ConstOperand

        from Ft.Ods.MetaData import Enumeration
        DbmMappings.g_metaKindToRepoClass[5] = Enumeration.Enumeration
        
        from Ft.Ods.MetaData import Exception
        DbmMappings.g_metaKindToRepoClass[6] = Exception.Exception

        from Ft.Ods.MetaData import Expression
        DbmMappings.g_metaKindToRepoClass[7] = Expression.Expression
        
        from Ft.Ods.MetaData import Interface
        DbmMappings.g_metaKindToRepoClass[8] = Interface.Interface

        from Ft.Ods.MetaData import Literal
        DbmMappings.g_metaKindToRepoClass[9] = Literal.Literal

        from Ft.Ods.MetaData import Member
        DbmMappings.g_metaKindToRepoClass[10] = Member.Member

        from Ft.Ods.MetaData import Module
        DbmMappings.g_metaKindToRepoClass[11] = Module.Module

        from Ft.Ods.MetaData import Operation
        DbmMappings.g_metaKindToRepoClass[12] = Operation.Operation

        from Ft.Ods.MetaData import Parameter
        DbmMappings.g_metaKindToRepoClass[13] = Parameter.Parameter

        from Ft.Ods.MetaData import PrimitiveType
        DbmMappings.g_metaKindToRepoClass[14] = PrimitiveType.PrimitiveType

        from Ft.Ods.MetaData import Relationship
        DbmMappings.g_metaKindToRepoClass[15] = Relationship.Relationship

        from Ft.Ods.MetaData import Repository
        DbmMappings.g_metaKindToRepoClass[16] = Repository.Repository

        from Ft.Ods.MetaData import Structure
        DbmMappings.g_metaKindToRepoClass[17] = Structure.Structure

        from Ft.Ods.MetaData import TypeDefinition
        DbmMappings.g_metaKindToRepoClass[18] = TypeDefinition.TypeDefinition

        from Ft.Ods.MetaData import Union
        DbmMappings.g_metaKindToRepoClass[19] = Union.Union

        from Ft.Ods.MetaData import UnionCase
        DbmMappings.g_metaKindToRepoClass[20] = UnionCase.UnionCase

    def _loadPythonClasses(self):
        #FIXME this could be optimized to only load when we need them
        rt = self._db['ftods_pythonclass'].keys()
        for id in rt:
            name = self._db['ftods_pythonclass'][id]
            self._pythonClasses[name] = int(id)


    #Repository Object Helpers
    def _4ods_createInterfaceStorage(self, interface):
        interface_name = 'interface__%i' % interface._4ods_getRepositoryId()
        self._db.createTable(interface_name)

        #See if we need a new extent
        if interface.meta_kind._v == 1:
            system = self._db.getTable('ftods_system')
            extents = self._db['ftods_extents']
            for extent in interface.extents._4ods_getContents():
                #We got one to add
                eid = system['ftods_extentid']
                system['ftods_extentid'] = eid + 1
                extents[extent] = (extent, eid, Constants.ObjectTypes.OBJECT)

    def _4ods_createDictionaryStorage(self,dict):
        raise "Not Supported"
        qstr = 'CREATE TABLE dictionary__%d (_lid INT8, keys ' % dict._4ods_getId()

        t = self._4ods_toSqlTypes((dict.key_type._4ods_getOdmgType(),))[0]
        qstr = qstr + t
        qstr = qstr + '[], values '
        t = self._4ods_toSqlTypes((dict.subtype._4ods_getOdmgType(),))[0]
        qstr = qstr + t
        qstr = qstr + '[])'
        self._db.execute(qstr)

    def _4ods_updateInterfaceStorage(self,ro):
        if ro._4ods_getRepositoryId() in self._newRepoObjects:
            return
        raise "Not Supported"

        #Here are a list of changes we need to look for
        #1 A property was added
        #2 A property was deleted
        #3 The list of properties was rearranged
        #4 A property was changed
        
        #What we will do is accumulate all of the changes, then
        #Change the class structure at the end
        #NOTE derived classes make this extremely difficult
        #NOTE keep all data we can!!!!
        

        #First we need to get a pictuire of how the class looks on disk
        #We must get this from the datastore as our in memory version is probably modified
        if ro.meta_kind == RepositoryObject.MetaKind.mk_class:
            tableName = 'ftods_class'
        else:
            tableName = 'ftods_interface'

        qstr = "SELECT defines from %s where _repoId = %i" %(tableName,ro._4ods_getRepositoryObjecId())
        rt = self._db.queryAllTuple(qstr)

        print rt

        #This code will look for added properties.

    def _4ods_deleteInterfaceStorage(self,ro):
        if ro.derives.cardinality():
            raise "Has derived class, cannot delete"
        if ro.meta_kind._v == 1 and ro.extensions.cardinality():
            raise "Has extensions, cannot delete"

        self._db.dropTable('interface__%i' % ro._4ods_getRepositoryId())

    def _4ods_makeTuple(self,id):
        if self._objects.has_key(id):
            o = self._objects[id]
            #Create from Memory
            t = (id, (o.tupleTypes,o.tupleNames,o._4ods_getFullTuple()))
        else:
            pClass = self._4ods_getObjectClass(id)
            repoId = pClass.typeId
            values = self._4ods_readObjectTuple(id,repoId,pClass.tupleTypes)
            t = (id, (pClass.tupleTypes,pClass.tupleNames,values))
        return t

    def _4ods_readObjectTuple(self,id,repoId,types):
        tableName = 'interface__%i' % repoId
        try:
            return self._db[tableName][str(id)]
        except:
            raise FtodsObjectNotFound(id)

    def _4ods_getObjectClass(self,id):
        rt = self._db['ftods_actualclass'].get(str(id))
        if rt:
            classId = rt[1]
            rt = self._objectClasses.get(classId)
            if rt is None:
                # Load the class
                rt = self._4ods_loadPythonClass(classId)
        return rt

    def _4ods_getCollectionClass(self,id):
        rt = self._db['ftods_collectionclass'].get(str(id))
        if rt is None:
            return rt
        classId = rt[1]
        subType = rt[2]
        collType = rt[3]
        klass = self._objectClasses.get(classId)
        if not klass:
            # Load the class
            klass = self._4ods_loadPythonClass(classId)
        return klass, subType, collType

    def _4ods_loadPythonClass(self, classId):
        rt = self._db['ftods_pythonclass'].get(str(classId))
        ps = string.split(rt[1], '.')
        curMod = __import__(string.join(ps[:-1],'.'))
        for p in ps[1:]:
            curMod = getattr(curMod,p)
        self._objectClasses[classId] = curMod
        return curMod
        
    def _4ods_nextId(self, sequence):
        system = self._db.getTable('ftods_system')
        table_name = 'ftods_%sid' % sequence
        id = system[table_name]
        system[table_name] = id + 1
        return id
        
    def _4ods_getPythonClassId(self, pClass):
        mods = string.split(pClass.__module__,'.')
        if mods[0] == '__main__':
            import sys
            mods[0] = string.join(string.split(sys.argv[0],'.')[:-1],'.')
        pName = string.join(mods,'.') + '.' + pClass.__name__
        if self._pythonClasses.has_key(pName):
            return self._pythonClasses[pName]
        pid = self._4ods_nextId('object')
        self._pythonClasses[pName] = pid
        self._db['ftods_pythonclass'][str(pid)] = (pid, pName)
        return pid

    #Misc Functions
    def _4ods_updateExtentsAdd(self,obj):
        oid = obj._4ods_getId()
        extents = self._db['ftods_extents']
        mapping = self._db.getTable('ftods_extentmapping')
        for name in obj._extents:
            eid = str(extents[name][1])
            items = mapping.get(eid, [])
            items.append(oid)
            mapping[eid] = items

    def _4ods_updateExtentsDelete(self,obj):
        oid = obj._4ods_getId()
        extents = self._db['ftods_extents']
        mapping = self._db['ftods_extentmapping']
        for name in obj._extents:
            eid = str(extents[name][1])
            items = mapping[str(extents[name][1])]
            items.remove(oid)
            mapping[eid] = items

    def _4ods_loadExtent(self, eid):
        return self._db['ftods_extentmapping'].get(str(eid), [])
