########################################################################
#
# File Name: 	    DbmManagement.py
#
# Documentation:	http://docs.4suite.org/4ODS/StorageManager/Adapters/DbmManagement.py.html
#
"""
The DBM Database Management functions
WWW: http://4suite.org/             e-mail: support@4suite.org

Copyright (c) 2000 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

import Util, Constants

import DbmMappings
from Ft.Lib import DbmDatabase

### Management Functions ###

def Connect(connectString):
    dbName = Util.SplitDbConnectString(connectString)[0]
    return DbmDatabase.Database('ftods__' + dbName)

def ExistsDb(connectString):
    dbName = 'ftods__' + Util.SplitDbConnectString(connectString)[0]
    return dbName in DbmDatabase.GetAllDatabaseNames()

def ClearDb(connectString):
    db = Connect(connectString)

    #Get all of the interfaces
    rids = db['ftods_class'].keys()
    rids.extend(db['ftods_interface'].keys())
    table_names = map(lambda r: "interface__%s" % r, rids)
    for table_name in table_names:
        table = db[table_name]
        for key in table.keys():
            del table[key]

    extents = db['ftods_extents'].keys()
    extentmapping = db['ftods_extentmapping']
    for name in extents:
        if name != 'ftods_types':
            del extentmapping[eid]

    binding = db['ftods_binding']
    for name in binding.keys():
        del binding[name]
    db.commit()

def CleanDb(connectString):
    pass

def DestroyDb(connectString):
    dbName = Util.SplitDbConnectString(connectString)[0]
    DbmDatabase.DropDatabase('ftods__' + dbName)

def CreateDb(connectString, initRepo=1):
    dbName = Util.SplitDbConnectString(connectString)[0]
    dbName = 'ftods__' + dbName
    db = DbmDatabase.CreateDatabase(dbName)
    system = db.createTable('ftods_system')

    # Metadata tables
    for sequence in DbmMappings.g_sequences.keys():
        system[sequence] = 1

    # Setup the tables
    for name in DbmMappings.g_sortedTableNames:
        db.createTable(name)

    # Create the collection tables
    for name in DbmMappings.g_collectionTableMapping.values():
        db.createTable(name)

    if not initRepo:
        return

    eid = system['ftods_extentid']
    system['ftods_extentid'] = eid + 1
    
    extents = db.getTable('ftods_extents')
    extents['ftods_types'] = ('ftods_types', eid, Constants.ObjectTypes.REPO)

    db.commit()
    
    from Ft.Ods import Database
    from Ft.Ods.MetaData import PrimitiveType, Repository
    db = Database.Database()
    db.open(connectString)
    tx = db.new()
    tx.begin()
    r = Repository.Repository()
    r.name = 'REPOSITORY'

    metaobject = r.add_module('ODLMetaObjects')
    odltypes = r.add_module('ODLTypes')

    for pk in PrimitiveType.PrimitiveKind.elements:
        t = r.create_primitive_type(pk)
        t.name = pk._n
        t.comment = "Automatically added by creation script"
        r.bind(t.name,t)

    tx.commit()
    db.close()

def GrantAccess(dbName, user, level):
    #FIXME: implement (copied from PostgresManagement)
    return
    #Get a table list
    db = Connect(dbName)

    #The table list depends on the level

    rTables = []
    wTables = []

    for table_name in PostgresMappings.g_tables.keys():
        if level == 'ALL':
            wTables.append(table_name)
        if level == 'METADATA':
            if PostgresMappings.g_tables[table_name][TABLE_ACCESS] == PostgresMappings.SYSTEM:
                wTables.append(table_name)
            if PostgresMappings.g_tables[table_name][TABLE_ACCESS] == PostgresMappings.METADATA:
                wTables.append(table_name)
            if PostgresMappings.g_tables[table_name][TABLE_ACCESS] == PostgresMappings.OBJECT:
                rTables.append(table_name)
        if level == 'OBJECT':
            if PostgresMappings.g_tables[table_name][TABLE_ACCESS] == PostgresMappings.SYSTEM:
                wTables.append(table_name)
            if PostgresMappings.g_tables[table_name][TABLE_ACCESS] == PostgresMappings.METADATA:
                rTables.append(table_name)
            if PostgresMappings.g_tables[table_name][TABLE_ACCESS] == PostgresMappings.OBJECT:
                wTables.append(table_name)
                
    for sequence_name in PostgresMappings.g_sequences.keys():
        if level == 'ALL':
            wTables.append(sequence_name)
        if level == 'METADATA':
            if PostgresMappings.g_sequences[sequence_name] == PostgresMappings.SYSTEM:
                wTables.append(sequence_name)
            if PostgresMappings.g_tables[sequence_name] == PostgresMappings.METADATA:
                wTables.append(sequence_name)
            if PostgresMappings.g_tables[sequence_name] == PostgresMappings.OBJECT:
                rTables.append(sequence_name)
        if level == 'OBJECT':
            if PostgresMappings.g_tables[sequence_name] == PostgresMappings.SYSTEM:
                wTables.append(sequence_name)
            if PostgresMappings.g_tables[sequence_name] == PostgresMappings.METADATA:
                rTables.append(sequence_name)
            if PostgresMappings.g_tables[sequence_name] == PostgresMappings.OBJECT:
                wTables.append(sequence_name)

    #Grant on the collection tables
    doneTables = []
    for table_type in PostgresMappings.g_collectionTableMapping.keys():
        for table_name in PostgresMappings.g_collectionTableMapping[table_type].values():
            if table_name in doneTables:
                continue
            doneTables.append(table_name)

            if level == 'ALL':
                wTables.append(table_name)
            if level == 'METADATA':
                rTables.append(sequence_name)
            if level == 'OBJECT':
                wTables.append(sequence_name)



    if level in ['ALL','OBJECT']:
        rt = db.queryAllTuple("SELECT _repoid from ftods_class")
        wTables = wTables + map(lambda x: "interface__%s" % x[0],rt)
        rt = db.queryAllTuple("SELECT _repoid from ftods_interface")
        wTables = wTables + map(lambda x: "interface__%s" % x[0],rt)

    for wAccess in ['SELECT','UPDATE','INSERT','DELETE']:
        for tName in wTables:
            qstr = "GRANT %s on %s to %s" % (wAccess,tName,user)
            db.execute(qstr)
    for tName in rTables:
        qstr = "GRANT SELECT on %s to %s" % (tName,user)
        db.execute(qstr)
