########################################################################
#
# File Name: 	        PostgresMappings.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/Drivers/PostgresMappings.py.html
#
"""
The Postgres Mappings for the backend
WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 1999 FourThought, Inc, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

from Ft.Ods.StorageManager.Adapters import Constants

g_collectionTableMapping = {Constants.CollectionTypes.LIST:'ftods_list',
                            Constants.CollectionTypes.BAG:'ftods_bag',
                            Constants.CollectionTypes.SET:'ftods_set',
                            }

g_metaKindToTableName = {0:'ftods_attribute',
                         1:'ftods_class',
                         2:'ftods_metadatacollection',
                         3:'ftods_constant',
                         4:'ftods_const_operand',
                         5:'ftods_enumeration',
                         6:'ftods_exception',
                         7:'ftods_expression',
                         8:'ftods_interface',
                         9:'ftods_metadataliteral',
                         10:'ftods_member',
                         11:'ftods_module',
                         12:'ftods_operation',
                         13:'ftods_parameter',
                         14:'ftods_primitive_type',
                         15:'ftods_relationship',
                         16:'ftods_module',
                         17:'ftods_structure',
                         18:'ftods_type_definition',
                         19:'ftods_union',
                         20:'ftods_union_case'
                         }

g_metaKindToRepoClass = {}

class SystemTypes:
    NAME=1
    OBJECT=2
    ENUMERATION = 3
    STRING = 4
    BOOLEAN = 5
    SHORT = 6

SYSTEM_ACCESS = 1
OBJECT_ACCESS = 2
METADATA_ACCESS = 3

TABLE_INIT = 1
TABLE_ACCESS = 2
TABLE_INHERITS = 3

PRIMARY_KEY = 1

#A list of all of the table names in creation order
g_sortedTableNames = ['ftods_binding',
                      'ftods_extents',
                      'ftods_extentmapping',
                      'ftods_metaname',
                      'ftods_pythonclass',
                      'ftods_actualclass',
                      'ftods_collectionclass',
                      'ftods_object',
                      'ftods_collection',
                      'ftods_literal',
                      'ftods_repomap',
                      'ftods_repositoryobject',
                      'ftods_metaobject',
                      'ftods_definingscope',
                      'ftods_module',
                      'ftods_operation',
                      'ftods_exception',
                      'ftods_constant',
                      'ftods_property',
                      'ftods_attribute',
                      'ftods_relationship',
                      'ftods_type',
                      'ftods_typedef',
                      'ftods_primitive_type',
                      'ftods_interface',
                      'ftods_class',
                      'ftods_metadatacollection',
                      'ftods_dictionary',
                      'ftods_enumeration',
                      'ftods_structure',
                      'ftods_union',
                      'ftods_specifier',
                      'ftods_member',
                      'ftods_unioncase',
                      'ftods_parameter',
                      'ftods_operand',
                      'ftods_metadataliteral',
                      'ftods_const_operand',
                      'ftods_expression',                      
                      ]
                      
g_tables = {
    'ftods_binding' : {'name' : 0, 'objid' : 1, 'repoid' : 2,
                       '__access' : OBJECT_ACCESS,
                       '__key' : 'name',
                       },

    'ftods_extents' : {'name' : 0, 'extentid' : 1, 'persistentType' : 2,
                       '__access' : SYSTEM_ACCESS,
                       '__key' : 'name',
                       },

    'ftods_extentmapping' : {'extentid' : 0, 'id' : 1,
                             '__access' : SYSTEM_ACCESS,
                             '__key' : None,
                             },
    
    'ftods_metaname' : {'repoid' : 0, 'meta_kind' : 1,
                        '__access' : METADATA_ACCESS,
                        '__key' : 'repoid',
                        },
    
    'ftods_pythonclass' : {'pythonClassId' : 0, 'pythonClassName' : 1,
                           '__access' : OBJECT_ACCESS,
                           '__key' : 'pythonClassId',
                           },
                          
    'ftods_actualclass' : {'oid' : 0, 'pythonClassId' : 1,
                           '__access' : OBJECT_ACCESS,
                           '__key' : 'oid',
                           },

    'ftods_collectionclass' : {'cid' : 0, 'pythonClassId' : 1, 'subtype' : 2,
                               '__access' : OBJECT_ACCESS,
                               '__key' : 'cid',
                               },

    'ftods_object' : {'temp' : 0,
                      '__access' : OBJECT_ACCESS,
                      '__key' : None,
                      },

    'ftods_collection' : {'cid' : 0,
                          '__access' : SYSTEM_ACCESS,
                          '__key' : 'cid',
                          },
                        
    'ftods_literal' : {'lid' : 0,
                       '__access' : OBJECT_ACCESS,
                       '__key' : 'lid',
                       },

    # Finish conversion from here
    'ftods_repomap':{TABLE_INIT:[('repoid',SystemTypes.OBJECT,'PRIMARY KEY'),
                                 ('meta_kind',SystemTypes.ENUMERATION,''),
                                 ],
                     TABLE_ACCESS:OBJECT_ACCESS,
                     TABLE_INHERITS:""
                     },
    'ftods_repositoryobject':{TABLE_INIT:[('_repoid',SystemTypes.OBJECT,'PRIMARY KEY'),
                                          ('meta_kind',SystemTypes.ENUMERATION,''),
                                          ],
                     TABLE_ACCESS:METADATA_ACCESS,
                     TABLE_INHERITS:""
                     },

    'ftods_metaobject':{TABLE_INIT:[('name',SystemTypes.STRING,''),
                                    ('comment',SystemTypes.STRING,''),
                                    ('definedIn',SystemTypes.OBJECT,''),
                                    ],
                     TABLE_ACCESS:METADATA_ACCESS,
                     TABLE_INHERITS:"ftods_repositoryobject"
                     },
                        
    'ftods_definingscope':{TABLE_INIT:[('defines',SystemTypes.OBJECT,''),
                                       ],
                     TABLE_ACCESS:METADATA_ACCESS,
                     TABLE_INHERITS:""
                     },
    'ftods_module':{TABLE_INIT:[
    ],
                     TABLE_ACCESS:METADATA_ACCESS,
                     TABLE_INHERITS:"ftods_metaobject,ftods_definingscope"
                     },
    'ftods_operation':{TABLE_INIT:[('signature',SystemTypes.OBJECT,''),
                                   ('result',SystemTypes.OBJECT,''),
                                   ('exceptions',SystemTypes.OBJECT,''),
                                   ],
                     TABLE_ACCESS:METADATA_ACCESS,
                     TABLE_INHERITS:"ftods_metaobject"
                     },
                       
    'ftods_exception':{TABLE_INIT:[('result',SystemTypes.OBJECT,''),
                                   ('operations',SystemTypes.OBJECT,''),
                                   ],
                     TABLE_ACCESS:METADATA_ACCESS,
                     TABLE_INHERITS:"ftods_metaobject"
                     },

    'ftods_constant':{TABLE_INIT:[('the_value',SystemTypes.OBJECT,''),
                                  ('type',SystemTypes.OBJECT,''),
                                  ('referenced_by',SystemTypes.OBJECT,''),
                                  ('enumeration',SystemTypes.OBJECT,''),
                                  ],
                     TABLE_ACCESS:METADATA_ACCESS,
                     TABLE_INHERITS:"ftods_metaobject"
                     },
                      
    'ftods_property':{TABLE_INIT:[('type',SystemTypes.OBJECT,''),
                                  ],
                     TABLE_ACCESS:METADATA_ACCESS,
                     TABLE_INHERITS:"ftods_metaobject"
                     },
                      

    'ftods_attribute':{TABLE_INIT:[('is_read_only',SystemTypes.BOOLEAN,''),
                                  ],
                     TABLE_ACCESS:METADATA_ACCESS,
                     TABLE_INHERITS:"ftods_property"
                     },
                      
    'ftods_relationship':{TABLE_INIT:[('traversal',SystemTypes.OBJECT,''),
                                  ],
                     TABLE_ACCESS:METADATA_ACCESS,
                     TABLE_INHERITS:"ftods_property"
                     },
                      
    'ftods_type':{TABLE_INIT:[('collections',SystemTypes.OBJECT,''),
                              ('dictionaries',SystemTypes.OBJECT,''),
                              ('specifiers',SystemTypes.OBJECT,''),
                              ('unions',SystemTypes.OBJECT,''),
                              ('operations',SystemTypes.OBJECT,''),
                              ('properties',SystemTypes.OBJECT,''),
                              ('constants',SystemTypes.OBJECT,''),
                              ('type_defs',SystemTypes.OBJECT,''),
                              ],
                     TABLE_ACCESS:METADATA_ACCESS,
                     TABLE_INHERITS:"ftods_metaobject"
                     },
                      
    'ftods_typedef':{TABLE_INIT:[('type',SystemTypes.OBJECT,''),
                                  ],
                     TABLE_ACCESS:METADATA_ACCESS,
                     TABLE_INHERITS:"ftods_type"
                     },
                      
    'ftods_primitive_type':{TABLE_INIT:[('primitive_kind',SystemTypes.ENUMERATION,''),
                                  ],
                            TABLE_ACCESS:METADATA_ACCESS,
                            TABLE_INHERITS:"ftods_type"
                            },
                      
    'ftods_interface':{TABLE_INIT:[('inherits',SystemTypes.OBJECT,''),
                                  ('derives',SystemTypes.OBJECT,''),
                                  ],
                     TABLE_ACCESS:METADATA_ACCESS,
                     TABLE_INHERITS:"ftods_type,ftods_definingscope"
                     },
                      
    'ftods_class':{TABLE_INIT:[('extents',SystemTypes.OBJECT,''),
                               ('keys',SystemTypes.OBJECT,''),
                               ('extender',SystemTypes.OBJECT,''),
                               ('extensions',SystemTypes.OBJECT,''),
                                  ],
                   TABLE_ACCESS:METADATA_ACCESS,
                   TABLE_INHERITS:"ftods_interface"
                   },
                      
    'ftods_metadatacollection':{TABLE_INIT:[('collection_kind',SystemTypes.ENUMERATION,''),
                                            ('max_size',SystemTypes.SHORT,''),
                                            ('subtype',SystemTypes.OBJECT,''),
                                            ],
                                TABLE_ACCESS:METADATA_ACCESS,
                                TABLE_INHERITS:"ftods_type"
                                },
                      
    'ftods_dictionary':{TABLE_INIT:[('key_type',SystemTypes.OBJECT,''),
                                    ],
                        TABLE_ACCESS:METADATA_ACCESS,
                        TABLE_INHERITS:"ftods_collection"
                        },
                      
    'ftods_enumeration':{TABLE_INIT:[('elements',SystemTypes.OBJECT,''),
                                     ],
                         TABLE_ACCESS:METADATA_ACCESS,
                         TABLE_INHERITS:"ftods_type"
                         },
                      
    'ftods_structure':{TABLE_INIT:[('fields',SystemTypes.OBJECT,''),
                                  ('exception',SystemTypes.OBJECT,''),
                                  ],
                       TABLE_ACCESS:METADATA_ACCESS,
                       TABLE_INHERITS:"ftods_type"
                       },
                      
    'ftods_union':{TABLE_INIT:[('switch_type',SystemTypes.OBJECT,''),
                               ('cases',SystemTypes.OBJECT,''),
                               ],
                   TABLE_ACCESS:METADATA_ACCESS,
                   TABLE_INHERITS:"ftods_metaobject"
                   },

    'ftods_specifier':{TABLE_INIT:[('name',SystemTypes.STRING,''),
                                   ('type',SystemTypes.OBJECT,''),
                                   ],
                       TABLE_ACCESS:METADATA_ACCESS,
                       TABLE_INHERITS:"ftods_repositoryobject"
                       },
                      
    'ftods_member':{TABLE_INIT:[('strucutre_type',SystemTypes.OBJECT,''),
                                ],
                    TABLE_ACCESS:METADATA_ACCESS,
                    TABLE_INHERITS:"ftods_specifier"
                    },
    'ftods_unioncase':{TABLE_INIT:[('_union',SystemTypes.OBJECT,''),
                                  ('caselabels',SystemTypes.OBJECT,''),
                                   ],
                     TABLE_ACCESS:METADATA_ACCESS,
                     TABLE_INHERITS:"ftods_specifier"
                     },
                      
    'ftods_parameter':{TABLE_INIT:[('parametermode',SystemTypes.ENUMERATION,''),
                                  ('operations',SystemTypes.OBJECT,''),
                                  ],
                     TABLE_ACCESS:METADATA_ACCESS,
                     TABLE_INHERITS:"ftods_specifier"
                     },
                      
    'ftods_operand':{TABLE_INIT:[('operand_in',SystemTypes.OBJECT,''),
                                  ('value_of',SystemTypes.OBJECT,''),
                                  ('size_of',SystemTypes.OBJECT,''),
                                  ('case_in',SystemTypes.OBJECT,''),
                                  ],
                     TABLE_ACCESS:METADATA_ACCESS,
                     TABLE_INHERITS:"ftods_repositoryobject"
                     },
                      
    'ftods_metadataliteral':{TABLE_INIT:[('_literal_value',SystemTypes.STRING,''),
                                         ],
                     TABLE_ACCESS:METADATA_ACCESS,
                     TABLE_INHERITS:"ftods_operand"
                     },
                      
    'ftods_const_operand':{TABLE_INIT:[('_references',SystemTypes.OBJECT,''),
                                      ],
                     TABLE_ACCESS:METADATA_ACCESS,
                     TABLE_INHERITS:"ftods_operand"
                     },
                      
    'ftods_expression':{TABLE_INIT:[('operator',SystemTypes.STRING,''),
                                  ('theoperands',SystemTypes.OBJECT,''),
                                  ],
                     TABLE_ACCESS:METADATA_ACCESS,
                     TABLE_INHERITS:"ftods_operand"
                     },
                      
    }


g_sequences = {'ftods_objectid':OBJECT_ACCESS,
               'ftods_repoid':METADATA_ACCESS,
               'ftods_litid':OBJECT_ACCESS,
               'ftods_extentid':OBJECT_ACCESS,
               'ftods_pythonclassid':SYSTEM_ACCESS,
               'ftods_collectionid':SYSTEM_ACCESS
               }
             
             

