########################################################################
#
# File Name: 	        OracleManagement.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/Drivers/OracleManagement.py.html
#
"""
The Oracle Database Management functions
WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 1999 FourThought, Inc, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

import string, sys

import OracleMappings
from Ft.Ods.StorageManager.Adapters import Util
from Ft.Ods.StorageManager.Adapters import Constants

import DCOracle

from DCOracle import ociUtil


def ClearDb(dbName):

    db = Connect(dbName)

    table_names = OracleMappings.g_tables.keys()

    #Get all of the interfaces
    rt = db.queryAllTuple('SELECT _repoid from ftods_class')
    for r in rt:
        table_names.append("interface__%s" % r[0])

    rt = db.queryAllTuple('SELECT _repoid from ftods_interface')
    for r in rt:
        table_names.append("interface__%s" % r[0])
        
    for table_name in table_names:
        db.execute("DELETE from %s" % table_name)


def CleanDb(dbName):
    pass


def DestroyDb(connString):


    db = Connect(connString)
    cursor = db.cursor()

    table_names = []
    cursor.execute("SELECT repoid from ftods_class")
    rt = cursor.fetchall()
    table_names = table_names + map(lambda x: "interface__%s" % x[0],rt)
    cursor.execute("SELECT repoid from ftods_interface")
    rt = cursor.fetchall()
    table_names = table_names + map(lambda x: "interface__%s" % x[0],rt)

    for table_name in table_names:
        try:
            db.execute('DROP TABLE %s' % table_name)
        except ociUtil.error, e:
            sys.stderr.write("Unable to drop table %s!\n" % table_name)
        

    for seq in OracleMappings.g_sequences.keys():
        try:
            db.execute('DROP SEQUENCE %s' % seq)
        except ociUtil.error, e:
            sys.stderr.write("Unable to drop sequence %s\n" % seq)
        

    for table_name in OracleMappings.g_sortedTableNames:
        try:
            db.execute('DROP TABLE %s' % table_name)
        except ociUtil.error, e:
            sys.stderr.write("Unable to drop table %s!\n" % table_name)

    #Create all of the collection tables
    doneTables = []
    for table_type in OracleMappings.g_collectionTableMapping.keys():
        for table_name in OracleMappings.g_collectionTableMapping[table_type].values():
            if table_name in doneTables:
                continue
            doneTables.append(table_name)
            try:
                db.execute('DROP TABLE %s' % table_name)
            except ociUtil.error, e:
                sys.stderr.write("Unable to drop table %s!\n" % table_name)



def TableInitString(table_name):
    initData = OracleMappings.g_tables[table_name][OracleMappings.TABLE_INIT]
    inherits = OracleMappings.g_tables[table_name][OracleMappings.TABLE_INHERITS]
    st = ' '
    if inherits:
        iTables = string.split(inherits,',')
        for parentTableName in iTables:
            st = st + TableInitString(parentTableName) + ','

    for data in initData:
        st = st + "%s %s %s," % (data[0],OracleMappings.g_systemTypesToSql[data[1]],data[2])
    st = st[:-1]
    return st



def CreateDb(connString,initRepo = 1):

    db = Connect(connString)

    for seq in OracleMappings.g_sequences.keys():
        db.execute('CREATE SEQUENCE %s' % seq)

    #NOTE: Check for validity
    if len(OracleMappings.g_sortedTableNames) != len(OracleMappings.g_tables.keys()):
        for table_name in OracleMappings.g_sortedTableNames:
            if table_name not in OracleMappings.g_tables.keys():
                print "%s in sorted but not dictionary" % table_name
        for table_name in OracleMappings.g_tables.keys():
            if table_name not in OracleMappings.g_sortedTableNames:
                print "%s in dictionary but not sorted" % table_name
        raise "Definition validity error"
        
    #Setup the tables
    for table_name in OracleMappings.g_sortedTableNames:
        st = "CREATE TABLE %s (" % table_name
        st = st + TableInitString(table_name) + ')'
        db.execute(st)

    #Create the collection containers
    valueTypeToSqlType = {
        'boolean':OracleMappings.g_odmgToSqlTypes[Constants.Types.BOOLEAN],
        'long':OracleMappings.g_odmgToSqlTypes[Constants.Types.UNSIGNED_LONG],
        'longlong':OracleMappings.g_odmgToSqlTypes[Constants.Types.SIGNED_LONG_LONG],
        'double':OracleMappings.g_odmgToSqlTypes[Constants.Types.DOUBLE],
        'string':OracleMappings.g_odmgToSqlTypes[Constants.Types.STRING]
        }

    #Create all of the collection tables
    doneTables = []
    for table_type in OracleMappings.g_collectionTableMapping.keys():
        for table_name in OracleMappings.g_collectionTableMapping[table_type].values():
            if table_name in doneTables:
                continue
            doneTables.append(table_name)
            fields = string.split(table_name,'_')
            valueTyp = fields[1]
            collTyp = fields[2]
            subType = valueTypeToSqlType[valueTyp]

            cString = 'CREATE TABLE %s (' % table_name
            iString = 'CREATE INDEX %s_index  on %s (' % (table_name,table_name)
            
            if collTyp == 'bag':
                cString = cString + 'cid NUMBER(30,0),ods_index NUMBER(15,0), value %s' % subType
                iString = iString + 'cid,ods_index'
            elif collTyp == 'list':
                cString = cString + 'cid NUMBER(30,0),ods_index NUMBER(15,0), value %s' % subType
                iString = iString + 'cid,ods_index'
            elif collTyp == 'set':
                cString = cString + 'cid NUMBER(30,0),ods_index NUMBER(15,0), value %s' % subType
                iString = iString + 'cid,ods_index'
            cString = cString + ')'
            iString = iString + ')'
            db.execute(cString)
            db.execute(iString)
        
    if not initRepo:
        db.commit()
        return

    #Create the initial structure

    cur = db.cursor()

    cur.execute("SELECT ftods_extentid.nextval from dual")
    id = int(cur.fetchall()[0][0])

    exts = ['ftods_typedefinition','ftods_primitive_type','ftods_interface','ftods_class','ftods_collection','ftods_dictionary','ftods_enumeration','ftods_structure','ftods_union']
                             
    db.execute("INSERT INTO ftods_extents (name,extentid,persistenttype) VALUES ('ftods_types',%s,%i)" % (id,Constants.ObjectTypes.REPO))

    for e in exts:
        db.execute("INSERT INTO ftods_extenttables (extentid,tableName) VALUES (%s,'%s')" % (id,e))

    db.commit()


    #Create the repository
    from Ft.Ods import Database
    from Ft.Ods.MetaData import PrimitiveType
    db = Database.Database()
    db.open(connString)
    tx = db.new()
    tx.begin()
    from Ft.Ods.MetaData import Repository
    r = Repository.Repository()
    r.name = 'REPOSITORY'

    metaobject = r.add_module('ODLMetaObjects')
    odltypes = r.add_module('ODLTypes')

    for pk in PrimitiveType.PrimitiveKind.elements:
        t = r.create_primitive_type(pk)
        t.name = pk._n
        t.comment = "Automatically added by creation script"
        r.bind(t.name,t)

    tx.commit()
    db.close()

def GrantAccess(dbName,user,level):

    #Get a table list
    db = Connect(dbName)

    #The table list depends on the level

    rTables = []
    wTables = []

    for table_name in OracleMappings.g_tables.keys():
        if level == 'ALL':
            wTables.append(table_name)
        if level == 'METADATA':
            if OracleMappings.g_tables[table_name][TABLE_ACCESS] == OracleMappings.SYSTEM:
                wTables.append(table_name)
            if OracleMappings.g_tables[table_name][TABLE_ACCESS] == OracleMappings.METADATA:
                wTables.append(table_name)
            if OracleMappings.g_tables[table_name][TABLE_ACCESS] == OracleMappings.OBJECT:
                rTables.append(table_name)
        if level == 'OBJECT':
            if OracleMappings.g_tables[table_name][TABLE_ACCESS] == OracleMappings.SYSTEM:
                wTables.append(table_name)
            if OracleMappings.g_tables[table_name][TABLE_ACCESS] == OracleMappings.METADATA:
                rTables.append(table_name)
            if OracleMappings.g_tables[table_name][TABLE_ACCESS] == OracleMappings.OBJECT:
                wTables.append(table_name)
                
    for sequence_name in OracleMappings.g_sequences.keys():
        if level == 'ALL':
            wTables.append(sequence_name)
        if level == 'METADATA':
            if OracleMappings.g_sequences[sequence_name] == OracleMappings.SYSTEM:
                wTables.append(sequence_name)
            if OracleMappings.g_tables[sequence_name] == OracleMappings.METADATA:
                wTables.append(sequence_name)
            if OracleMappings.g_tables[sequence_name] == OracleMappings.OBJECT:
                rTables.append(sequence_name)
        if level == 'OBJECT':
            if OracleMappings.g_tables[sequence_name] == OracleMappings.SYSTEM:
                wTables.append(sequence_name)
            if OracleMappings.g_tables[sequence_name] == OracleMappings.METADATA:
                rTables.append(sequence_name)
            if OracleMappings.g_tables[sequence_name] == OracleMappings.OBJECT:
                wTables.append(sequence_name)

    if level in ['ALL','OBJECT']:
        rt = db.queryAllTuple("SELECT _repoid from ftods_class")
        wTables = wTables + map(lambda x: "interface__%s" % x[0],rt)
        rt = db.queryAllTuple("SELECT _repoid from ftods_interface")
        wTables = wTables + map(lambda x: "interface__%s" % x[0],rt)

    for wAccess in ['SELECT','UPDATE','INSERT','DELETE']:
        for tName in wTables:
            qstr = "GRANT %s on %s to %s" % (wAccess,tName,user)
            db.execute(qstr)
    for tName in rTables:
        qstr = "GRANT SELECT on %s to %s" % (tName,user)
        db.execute(qstr)
    
def Connect(connectString):
    #Split the connection string
    dbName,hostName,port,userName,passwd = Util.SplitDbConnectString(connectString)
    oracleConnect = "%s/%s@%s" % (userName,passwd,dbName)
    db = DCOracle.Connect(oracleConnect)
    return db

