########################################################################
#
# File Name: 	        PostgresHelper.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/Drivers/PostgresHelper.py.html
#
"""
The Postgres Helper Functions back-end.
WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 1999 FourThought, Inc, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

import string

from Ft.Ods.StorageManager.Adapters import Constants

import PostgresMappings

from Ft.Ods import FtodsBindingException
from Ft.Ods import FtodsObjectNotFound
from Ft.Lib.PgDatabase import EscapeQuotes


class PostgresHelper:

    def _init(self):
        self._importMetaData()
        self._loadPythonClasses()
        

    #Init Helpers
    def _importMetaData(self):
        #FIXME this could be optimized to only load if/when we need it!
        if len(PostgresMappings.g_metaKindToRepoClass.keys()):
            return

        from Ft.Ods.MetaData import Attribute
        PostgresMappings.g_metaKindToRepoClass[0] = Attribute.Attribute

        from Ft.Ods.MetaData import Class
        PostgresMappings.g_metaKindToRepoClass[1] = Class.Class

        from Ft.Ods.MetaData import Collection
        PostgresMappings.g_metaKindToRepoClass[2] = Collection.Collection

        from Ft.Ods.MetaData import Constant
        PostgresMappings.g_metaKindToRepoClass[3] = Constant.Constant

        from Ft.Ods.MetaData import ConstOperand
        PostgresMappings.g_metaKindToRepoClass[4] = ConstOperand.ConstOperand

        from Ft.Ods.MetaData import Enumeration
        PostgresMappings.g_metaKindToRepoClass[5] = Enumeration.Enumeration
        
        from Ft.Ods.MetaData import Exception
        PostgresMappings.g_metaKindToRepoClass[6] = Exception.Exception

        from Ft.Ods.MetaData import Expression
        PostgresMappings.g_metaKindToRepoClass[7] = Expression.Expression
        
        from Ft.Ods.MetaData import Interface
        PostgresMappings.g_metaKindToRepoClass[8] = Interface.Interface

        from Ft.Ods.MetaData import Literal
        PostgresMappings.g_metaKindToRepoClass[9] = Literal.Literal

        from Ft.Ods.MetaData import Member
        PostgresMappings.g_metaKindToRepoClass[10] = Member.Member

        from Ft.Ods.MetaData import Module
        PostgresMappings.g_metaKindToRepoClass[11] = Module.Module

        from Ft.Ods.MetaData import Operation
        PostgresMappings.g_metaKindToRepoClass[12] = Operation.Operation

        from Ft.Ods.MetaData import Parameter
        PostgresMappings.g_metaKindToRepoClass[13] = Parameter.Parameter

        from Ft.Ods.MetaData import PrimitiveType
        PostgresMappings.g_metaKindToRepoClass[14] = PrimitiveType.PrimitiveType

        from Ft.Ods.MetaData import Relationship
        PostgresMappings.g_metaKindToRepoClass[15] = Relationship.Relationship

        from Ft.Ods.MetaData import Repository
        PostgresMappings.g_metaKindToRepoClass[16] = Repository.Repository

        from Ft.Ods.MetaData import Structure
        PostgresMappings.g_metaKindToRepoClass[17] = Structure.Structure

        from Ft.Ods.MetaData import TypeDefinition
        PostgresMappings.g_metaKindToRepoClass[18] = TypeDefinition.TypeDefinition

        from Ft.Ods.MetaData import Union
        PostgresMappings.g_metaKindToRepoClass[19] = Union.Union

        from Ft.Ods.MetaData import UnionCase
        PostgresMappings.g_metaKindToRepoClass[20] = UnionCase.UnionCase


    def _loadPythonClasses(self):
        #FIXME this could be optimized to only load when we need them
        rt = self._db.queryAllTuple("SELECT * from ftods_pythonclass")
        for r in rt:
            self._pythonClasses[r[1]] = int(r[0])


    #Repository Object Helpers
    def _4ods_createInterfaceStorage(self,interface):

        qstr = 'CREATE TABLE interface__%i ( ' % interface._4ods_getRepositoryId()
        for d in interface.defines._4ods_getContents():
            if d.meta_kind._v == 12:
                #A refernece to some python code?????
                raise "We don;t yet know what to do with operations!!!!!!"
            elif d.meta_kind._v in [17,3]:
                #We don't do any thing with structus here
                pass
            else:
                t = self._4ods_toSqlTypes((d.type._4ods_getOdmgType(),))[0]
                qstr = qstr + 'def__%i %s,' % (d._4ods_getRepositoryId(),t)
        qstr = qstr[:-1] + ' ) INHERITS ( '
        bClasses = []
        for inh in interface.inherits._4ods_getContents():
            bClasses.append('interface__%i' % inh._4ods_getRepositoryId())
        if interface.meta_kind._v == 1 and interface.extender:
            bClasses.append('interface__%s' % interface.extender._4ods_getRepositoryId())
        if len(bClasses) == 0:
            bClasses.append('ftods_object')

        for bc in bClasses:
            qstr = qstr + bc + ','

        qstr = qstr [:-1]
        qstr = qstr + ' )'
        self._db.execute(qstr)

        #See if we need a new extent
        if interface.meta_kind._v == 1:
            for extent in interface.extents._4ods_getContents():
                #We got one to add
                id = self._db.queryAllTuple("SELECT NEXTVAL('ftods_extentid')")[0][0]
                self._db.execute("INSERT into ftods_extents (name,extentid,persistenttype) VALUES ('%s',%d,%d)" % (extent,id,Constants.ObjectTypes.OBJECT))
                

    def _4ods_createDictionaryStorage(self,dict):
        raise "Not Supported"
        qstr = 'CREATE TABLE dictionary__%d (_lid INT8, keys ' % dict._4ods_getId()

        t = self._4ods_toSqlTypes((dict.key_type._4ods_getOdmgType(),))[0]
        qstr = qstr + t
        qstr = qstr + '[], values '
        t = self._4ods_toSqlTypes((dict.subtype._4ods_getOdmgType(),))[0]
        qstr = qstr + t
        qstr = qstr + '[])'
        self._db.execute(qstr)
        
        



    def _4ods_updateInterfaceStorage(self,ro):
        if ro._4ods_getRepositoryId() in self._newRepoObjects:
            return
        raise "Not Supported"

        #Here are a list of changes we need to look for
        #1 A property was added
        #2 A property was deleted
        #3 The list of properties was rearranged
        #4 A property was changed
        
        #What we will do is accumulate all of the changes, then
        #Change the class structure at the end
        #NOTE derived classes make this extremely difficult
        #NOTE keep all data we can!!!!
        

        #First we need to get a pictuire of how the class looks on disk
        #We must get this from the datastore as our in memory version is probably modified
        if ro.meta_kind == RepositoryObject.MetaKind.mk_class:
            tableName = 'ftods_class'
        else:
            tableName = 'ftods_interface'

        qstr = "SELECT defines from %s where _repoId = %i" %(tableName,ro._4ods_getRepositoryObjecId())
        rt = self._db.queryAllTuple(qstr)

        print rt


        #This code will look for added properties.


    def _4ods_deleteInterfaceStorage(self,ro):
        
        if ro.derives.cardinality():
            raise "Has derived class, cannot delete"
        if ro.meta_kind._v == 1 and ro.extensions.cardinality():
            raise "Has extensions, cannot delete"

        
        qstr = 'DROP TABLE interface__%i' % ro._4ods_getRepositoryId()
        self._db.execute(qstr)


    def _4ods_makeTuple(self,id):


        if self._objects.has_key(id):
            o = self._objects[id]
            #Create from Memory
            t = (id, (o.tupleTypes,o.tupleNames,o._4ods_getFullTuple()))
        else:
            pClass = self._4ods_getObjectClass(id)
            repoId = pClass.typeId
            values = self._4ods_readObjectTuple(id,repoId,pClass.tupleTypes)
            t = (id, (pClass.tupleTypes,pClass.tupleNames,values))
        return t


    def _4ods_readObjectTuple(self,id,repoId,types):
        tableName = 'interface__%i' % repoId
        qstr = "SELECT * from %s where oid = %i"%(tableName,id)
        tup = self._db.queryAllTuple(qstr)
        if len(tup) == 0:
            raise FtodsObjectNotFound(id)
        tup = tup[0]
        tup = self._4ods_toOdmgValues(tup,types)
        return tup

    def _4ods_getObjectClass(self,id):
        qstr = 'SELECT pythonclassid from ftods_actualclass where _oid = %i'%id
        rt = self._db.queryAllTuple(qstr)
        if len(rt) == 0:
            return None
        pClassId = rt[0][0]

        if self._objectClasses.has_key(pClassId):
            return self._objectClasses[pClassId]

        #Load the class
        return self._4ods_loadPythonClass(pClassId)

    def _4ods_getCollectionClass(self,id):
        qstr = 'SELECT pythonclassid,subtype from ftods_collectionclass where _cid = %i'%id
        rt = self._db.queryAllTuple(qstr)
        if len(rt) == 0:
            return None
        pClassId = rt[0][0]
        subType = rt[0][1]

        if self._objectClasses.has_key(pClassId):
            return self._objectClasses[pClassId],subType
        return self._4ods_loadPythonClass(pClassId),subType

    def _4ods_loadPythonClass(self,pClassId):
        #Load the class
        rt = self._db.queryAllTuple('SELECT pythonclassname from ftods_pythonclass where pythonclassid = %s'%pClassId)
        ps = string.split(rt[0][0],'.')
        curMod = __import__(string.join(ps[:-1],'.'))
        for p in ps[1:]:
            curMod = getattr(curMod,p)
        self._objectClasses[pClassId] = curMod
        return curMod

        
    def _4ods_nextObjectId(self):
        return self._db.queryAllTuple("SELECT NEXTVAL('ftods_objid')")[0][0]

    def _4ods_getPythonClassId(self,pClass):
        mods = string.split(pClass.__module__,'.')
        if mods[0] == '__main__':
            import sys
            mods[0] = string.join(string.split(sys.argv[0],'.')[:-1],'.')
        pName = string.join(mods,'.') + '.' + pClass.__name__
        if self._pythonClasses.has_key(pName):
            return self._pythonClasses[pName]
        pId = self._db.queryAllTuple("SELECT NEXTVAL('ftods_pythonclassid')")[0][0]
        self._pythonClasses[pName] = pId
        self._db.execute("INSERT into ftods_pythonclass (pythonclassid,pythonclassname) VALUES(%i,'%s')" % (pId,pName))
        return pId

    #Misc Functions
    def _4ods_updateExtentsAdd(self,obj):
        for e in obj._extents:
            rt = self._db.queryAllTuple("SELECT extentid from ftods_extents where name = '%s'"%e)
            for eid in rt:
                eid = eid[0]
                self._db.execute('INSERT into ftods_extentmapping (extentid,id) VALUES (%s,%s)' % (eid,obj._4ods_getId()))

    def _4ods_updateExtentsDelete(self,obj):
        for e in obj._extents:
            rt = self._db.queryAllTuple("SELECT extentid from ftods_extents where name = '%s'"%e)
            for eid in rt:
                eid = eid[0]
                self._db.execute('DELETE FROM ftods_extentmapping WHERE extentid = %s and id = %d' % (eid,obj._4ods_getId()))

    def _4ods_loadExtent(self,extentId):
        rt = self._db.queryAllTuple('SELECT id from ftods_extentmapping where extentid = %s' % extentId)
        return map(lambda x: int(x[0]),rt)

    def _4ods_tupleToUpdateString(self,types,names,values):
        values = self._4ods_toSqlValues(values,types)
        qstr = ''
        for ctr in range(len(names)):
            if values[ctr] != None:
                if names[ctr] == '_oid':
                    continue
                qstr = qstr + names[ctr] + '=' + values[ctr] + ','

        return qstr[:-1]


    def _4ods_toSqlTypes(self,types):
        return tuple(map(lambda x,m=PostgresMappings.g_odmgToSqlTypes: m[x],types))

    def _4ods_toSqlValues(self,values,types,stringMod = 0):
        rt = []
        oTypes = Constants.Types
        lists = Constants.g_listTypes
        for ctr in range(len(values)):
            v = values[ctr]
            if v == None:
                rt.append(None)
                continue
            t = types[ctr]
            if t in [oTypes.STRING,oTypes.FILE]:
                if not stringMod % 2:
                    rt.append("'%s'" % EscapeQuotes(v))
                else:
                    rt.append('"%s"' % EscapeQuotes(v))
            elif t in [oTypes.UNSIGNED_SHORT,oTypes.OBJECT,oTypes.ENUMERATION,oTypes.SIGNED_SHORT]:
                rt.append("%i" % v)
            elif t in [oTypes.DOUBLE,oTypes.FLOAT]:
                rt.append("%lf" % v)
            elif t in [oTypes.UNSIGNED_LONG, oTypes.SIGNED_LONG_LONG, oTypes.SIGNED_LONG]:
                st = str(v)
                if st[-1] == 'L':
                    st = st [:-1]
                rt.append(st)
            elif t in [oTypes.BOOLEAN]:
                if not stringMod % 2:
                    rt.append( (v and "'t'") or "'f'")
                else:
                    rt.append( (v and '"t"') or '"f"')
            elif lists[t]:
                rt.append("%i" % v)
            else:
                raise "Unknown Type %s" % t
        return rt

    def _4ods_toOdmgValues(self,values,types):
        rt = []
        oTypes = Constants.Types
        lists = Constants.g_listTypes

        for ctr in range(len(values)):
            v = values[ctr]
            t = types[ctr]
            if t in [oTypes.UNSIGNED_SHORT,oTypes.OBJECT,oTypes.ENUMERATION,oTypes.SIGNED_SHORT]:
                if not v:
                    rt.append(0)
                else:
                    rt.append(int(v))
	    elif t in [oTypes.UNSIGNED_LONG,oTypes.SIGNED_LONG_LONG,oTypes.SIGNED_LONG]:
		rt.append(long(v))
	    elif t in [oTypes.DOUBLE,oTypes.FLOAT]:
		rt.append(float(v))
            elif t in [oTypes.STRING,oTypes.FILE]:
                rt.append(v)
            elif t in [oTypes.BOOLEAN]:
                rt.append( (v=='t'))
            elif lists[t]:
                if v:
                    rt.append(int(v))
                else:
                    rt.append(None)
            else:
                raise "Unknown Type %s" % t
        return rt












