########################################################################
#
# File Name: 	        PostgresManagement.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/Drivers/PostgresManagement.py.html
#
"""
The Postgres Database Management functions
WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 1999 FourThought, Inc, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

import string,os

import PostgresMappings
from Ft.Ods.StorageManager.Adapters import Util
from Ft.Ods.StorageManager.Adapters import Constants
from Ft.Lib import PgDatabase

def ExistsDb(connString):
    dbName,hostName,port,userName,passwd = Util.SplitDbConnectString(connString)
    pg_db_name = "ftods__" + dbName
    tmpl_db = PgDatabase.Database('template1',hostName,port,userName,passwd)
    rt = tmpl_db.query("SELECT datname FROM pg_database where datname='%s'" % pg_db_name)
    return len(rt) != 0


def ClearDb(dbName):

    db = Connect(dbName)

    #table_names = PostgresMappings.g_tables.keys()
    table_names = []

    #Get all of the interfaces
    rt = db.queryAllTuple('SELECT _repoid from ftods_class')
    for r in rt:
        table_names.append("interface__%s" % r[0])

    rt = db.queryAllTuple('SELECT _repoid from ftods_interface')
    for r in rt:
        table_names.append("interface__%s" % r[0])
        
    for table_name in table_names:
        db.execute("DELETE from %s" % table_name)



    rt = db.queryAllTuple('SELECT name,extentid from ftods_extents')

    for name,id in rt:
        if name == 'ftods_types':
            continue
        db.execute("DELETE from ftods_extentmapping where extentid = '%s'"%id)

    db.execute("DELETE from ftods_binding")

    TEMP_DIR = os.environ.get("FTODS_LARGE_FILE_DIR",'/var/local/data/ftods/')
    os.system("rm -rf %s/*" % TEMP_DIR)

def CleanDb(dbName):
    pass


def DestroyDb(dbName):

    dbName,hostName,port,userName,passwd = Util.SplitDbConnectString(dbName)
    dbName = 'ftods__' + dbName
    tmpl_db = PgDatabase.Database('template1',hostName,port,userName,passwd)    
    tmpl_db.query('drop database ' + dbName)


def CreateDb(connString,initRepo = 1):

    dbName,hostName,port,userName,passwd = Util.SplitDbConnectString(connString)
    tmpl_db = PgDatabase.Database('template1',hostName,port,userName,passwd)
    tmpl_db.query('create database ftods__' + dbName)

    pg_db = Connect(connString)

    #Metadata tables
    for seq in PostgresMappings.g_sequences.keys():
        pg_db.query('CREATE SEQUENCE %s' % seq)


    #NOTE: Check for validity
    if len(PostgresMappings.g_sortedTableNames) != len(PostgresMappings.g_tables.keys()):
        for table_name in PostgresMappings.g_sortedTableNames:
            if table_name not in PostgresMappings.g_tables.keys():
                print "%s in sorted but not dictionary" % table_name
        for table_name in PostgresMappings.g_tables.keys():
            if table_name not in PostgresMappings.g_sortedTableNames:
                print "%s in dictionary but not sorted" % table_name
        raise "Definition validity error"
        

    #Setup the tables
    for table_name in PostgresMappings.g_sortedTableNames:
        initData = PostgresMappings.g_tables[table_name][PostgresMappings.TABLE_INIT]
        inherits = PostgresMappings.g_tables[table_name][PostgresMappings.TABLE_INHERITS]
        st = "CREATE TABLE %s (" % table_name
        for data in initData:
            st = st + "%s %s %s," % (data[0],PostgresMappings.g_systemTypesToSql[data[1]],data[2])
        if len(initData):
            st = st[:-1] + ')'
        else:
            st = st + ')'
            
        if inherits:
            st = st + 'INHERITS (%s)' % inherits
        pg_db.execute(st)

    #Create the collection containers
    valueTypeToSqlType = {
        'boolean':PostgresMappings.g_odmgToSqlTypes[Constants.Types.BOOLEAN],
        'long':PostgresMappings.g_odmgToSqlTypes[Constants.Types.UNSIGNED_LONG],
        'longlong':PostgresMappings.g_odmgToSqlTypes[Constants.Types.SIGNED_LONG_LONG],
        'double':PostgresMappings.g_odmgToSqlTypes[Constants.Types.DOUBLE],
        'string':PostgresMappings.g_odmgToSqlTypes[Constants.Types.STRING]
        }

    #Create all of the collection tables
    doneTables = []
    for table_type in PostgresMappings.g_collectionTableMapping.keys():
        for table_name in PostgresMappings.g_collectionTableMapping[table_type].values():
            if table_name in doneTables:
                continue
            doneTables.append(table_name)
            fields = string.split(table_name,'_')
            valueTyp = fields[1]
            collTyp = fields[2]
            subType = valueTypeToSqlType[valueTyp]

            cString = 'CREATE TABLE %s (' % table_name
            iString = 'CREATE UNIQUE INDEX %s_index  on %s (' % (table_name,table_name)
            
            if collTyp == 'bag':
                cString = cString + 'index INT4, value %s' % subType
                iString = iString + '_cid,index'
            elif collTyp == 'list':
                cString = cString + 'index INT4, value %s' % subType
                iString = iString + '_cid,index'
            elif collTyp == 'set':
                cString = cString + 'index INT4, value %s' % subType
                iString = iString + '_cid,index'
            cString = cString + ') INHERITS (ftods_collection)'
            iString = iString + ')'
            pg_db.query(cString)
            pg_db.query(iString)
        
    if not initRepo:
        return

    #Create the initial structure

    id = pg_db.queryAllTuple("SELECT NEXTVAL('ftods_extentid')")[0][0]

    pg_db.execute("INSERT INTO ftods_extents (name,extentid,persistenttype) VALUES ('ftods_types',%s,%i)" % (id,Constants.ObjectTypes.REPO))

    #Create the repository
    from Ft.Ods import Database
    from Ft.Ods.MetaData import PrimitiveType
    db = Database.Database()
    db.open(connString)
    tx = db.new()
    tx.begin()
    from Ft.Ods.MetaData import Repository
    r = Repository.Repository()
    r.name = 'REPOSITORY'

    metaobject = r.add_module('ODLMetaObjects')
    odltypes = r.add_module('ODLTypes')

    for pk in PrimitiveType.PrimitiveKind.elements:
        t = r.create_primitive_type(pk)
        t.name = pk._n
        t.comment = "Automatically added by creation script"
        r.bind(t.name,t)

    tx.commit()
    db.close()

def GrantAccess(dbName,user,level):

    #Get a table list
    db = Connect(dbName)

    #The table list depends on the level

    rTables = []
    wTables = []

    for table_name in PostgresMappings.g_tables.keys():
        if level == 'ALL':
            wTables.append(table_name)
        if level == 'METADATA':
            if PostgresMappings.g_tables[table_name][TABLE_ACCESS] == PostgresMappings.SYSTEM:
                wTables.append(table_name)
            if PostgresMappings.g_tables[table_name][TABLE_ACCESS] == PostgresMappings.METADATA:
                wTables.append(table_name)
            if PostgresMappings.g_tables[table_name][TABLE_ACCESS] == PostgresMappings.OBJECT:
                rTables.append(table_name)
        if level == 'OBJECT':
            if PostgresMappings.g_tables[table_name][TABLE_ACCESS] == PostgresMappings.SYSTEM:
                wTables.append(table_name)
            if PostgresMappings.g_tables[table_name][TABLE_ACCESS] == PostgresMappings.METADATA:
                rTables.append(table_name)
            if PostgresMappings.g_tables[table_name][TABLE_ACCESS] == PostgresMappings.OBJECT:
                wTables.append(table_name)
                
    for sequence_name in PostgresMappings.g_sequences.keys():
        if level == 'ALL':
            wTables.append(sequence_name)
        if level == 'METADATA':
            if PostgresMappings.g_sequences[sequence_name] == PostgresMappings.SYSTEM:
                wTables.append(sequence_name)
            if PostgresMappings.g_tables[sequence_name] == PostgresMappings.METADATA:
                wTables.append(sequence_name)
            if PostgresMappings.g_tables[sequence_name] == PostgresMappings.OBJECT:
                rTables.append(sequence_name)
        if level == 'OBJECT':
            if PostgresMappings.g_tables[sequence_name] == PostgresMappings.SYSTEM:
                wTables.append(sequence_name)
            if PostgresMappings.g_tables[sequence_name] == PostgresMappings.METADATA:
                rTables.append(sequence_name)
            if PostgresMappings.g_tables[sequence_name] == PostgresMappings.OBJECT:
                wTables.append(sequence_name)

    #Grant on the collection tables
    doneTables = []
    for table_type in PostgresMappings.g_collectionTableMapping.keys():
        for table_name in PostgresMappings.g_collectionTableMapping[table_type].values():
            if table_name in doneTables:
                continue
            doneTables.append(table_name)

            if level == 'ALL':
                wTables.append(table_name)
            if level == 'METADATA':
                rTables.append(sequence_name)
            if level == 'OBJECT':
                wTables.append(sequence_name)



    if level in ['ALL','OBJECT']:
        rt = db.queryAllTuple("SELECT _repoid from ftods_class")
        wTables = wTables + map(lambda x: "interface__%s" % x[0],rt)
        rt = db.queryAllTuple("SELECT _repoid from ftods_interface")
        wTables = wTables + map(lambda x: "interface__%s" % x[0],rt)

    for wAccess in ['SELECT','UPDATE','INSERT','DELETE']:
        for tName in wTables:
            qstr = "GRANT %s on %s to %s" % (wAccess,tName,user)
            db.execute(qstr)
    for tName in rTables:
        qstr = "GRANT SELECT on %s to %s" % (tName,user)
        db.execute(qstr)
    
def Connect(connectString):
    #Split the connection string
    dbName,hostName,port,userName,passwd = Util.SplitDbConnectString(connectString)
    dbName = 'ftods__' + dbName
    return PgDatabase.Database(dbName,hostName,port,userName,passwd)    
