############################################################################
#
# File Name: 		ExtentManager.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/StorageManager/ExtentManager.py/html
#

#Manages Extents
#WWW: http://4suite.com/4ODS         e-mail: support@4suite.com
#
#Copyright (c) 2000 Fourthought Inc, USA.   All Rights Reserved.
#See  http://4suite.com/COPYRIGHT  for license and copyright information


#NOTE Do not inherit from CollectionBase becuase this is a readonly collection (can be rewritten much simplier)
class Extent:
    def __init__(self,eManager,name,extentId,pType):
        self.__extentId = extentId
        self.__pType = pType
        self.__eManager = eManager
        self.__name = name

        self.__oldObjects = []
        self.__oldObjectIds = []
        self.__newObjects = []

        self.__loaded = 0

    def cardinality(self):
        self.__updateObjects()
        return len(self.__oldObjectIds + self.__newObjects)

    def is_empty(self):
        return self.cardinality() == 0

    def is_ordered(self):
        return 0

    def allows_duplicates(self):
        return 0

    def contains_element(self, e):
        self.__updateObjects()
        if e._4ods_getId():
            return e._4ods_getId() in self.__oldObjectIds
        return e in self.__newObjects

    def create_iterator(self, stable):
        return Iterator.Iterator(self, stable)

    def create_bidirectional_iterator(self, stable):
        return BidirectionalIterator.BidirectionalIterator(self, stable)

    def select_element(self,OQL_predicate):
        raise FtodsNotSupportedException("select_element")

    def select(self,OQL_predicate):
        raise FtodsNotSupportedException("select")

    def query(self,OQL_predicate):
        raise FtodsNotSupportedException("select_element")

    def exists_element(self,OQL_predicate):
        raise FtodsNotSupportedException("select_element")


    #The pythonic interface
    def __len__(self): return self.cardinality()
    def __getitem__(self, i):
        self.__updateObjects()
        if i >= self.cardinality():
            raise IndexError(i)
        return self._4ods_retrieveElementByIndex(i)
    def __getslice__(self, i, j):
        self.__updateObjects()
        i = max(i, 0); j = max(j, 0)
        return self._4ods_getContents()[i:j]

    def pop(self, i=-1): return self._4ods_retrieveElementByIndex(i)

    def count(self, item): return list(self._4ods_getContents()).count(item)

    def index(self, item): return list(self._4ods_getContents()).index(item)


    def _4ods_getContents(self):
        self.__updateObjects()

        from Ft.Ods import Database
        db = Database.Database()
        self.__oldObjects = map(lambda i,o,d = db,t = self.__pType:
                                o and o or d._4ods_getPersistentObjectById(t,i),
                                self.__oldObjectIds,
                                self.__oldObjects)
        return self.__oldObjects + self.__newObjects

    def _4ods_retrieveElementByIndex(self,index):
        if index >= len(self.__oldObjects):
            return self.__newObjects[index - len(self.__oldObjects)]
        if self.__oldObjects[index] == None:
            from Ft.Ods import Database
            db = Database.Database()
            self.__oldObjects[index] = db._4ods_getPersistentObjectById(self.__pType,self.__oldObjectIds[index])
        return self.__oldObjects[index]

    def __updateObjects(self):

        if not self.__loaded:
            from Ft.Ods import Database
            db = Database.Database()
            ids = db._4ods_loadExtent(self.__extentId)
            self.__oldObjectIds = ids
            self.__oldObjects = [None] * len(ids)
            self.__loaded = 1

        self.__newObjects = filter(lambda x:x._4ods_getId() == 0,self.__eManager._4ods_getNewObjects(self.__name))
                
        delObjects = self.__eManager._4ods_getDeleteObjects(self.__name)
        for obj in delObjects:
            if obj._4ods_getId() and obj._4ods_getId() in self.__oldObjectIds:
                index = self.__oldObjectIds.index(obj._4ods_getId())
                del self.__oldObjects[index]
                del self.__oldObjectIds[index]
            else:
                if obj in self.__newObjects:
                    index = self.__newObjects.index(obj)
                    del self.__newObjects[index]


    def _4ods_getOid(self):
        return self.__extentId
    def _4ods_getId(self):
        return self.__extentId

    def _4ods_toTuple(self):
        #What is a tuple????
        #((ID,(attrTypes,attrNames,attrValues)),)
        return map(lambda x,f=self._4ods_makeTuple: f(x),self._4ods_getContents())

    def _4ods_makeTuple(self,o):
        t = (o._4ods_getId(), (o.tupleTypes,o.tupleNames,o._4ods_getFullTuple()))
        return t



    readOnlyAttributes = ()
    readWriteAttributes = ()
    relationships = ()

    tupleNames = ()
    tupleTypes = ()
    tupleIds = ()

    _extents = ()
    localExtents = ()


    _collections = ()
    collectionMap = {}
