############################################################################
#
# File Name: 		OifAdapter.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/StorageManager/OifAdapter.py/html
#
"""
Manages Oif input and Output
WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 2000 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

import string

from Ft.Ods.Parsers.Oif import OifParseTreeMapping

from Ft.Ods.StorageManager.Adapters import Constants


class OifAdapter:
    def __init__(self):
        self.__tags = {}
    

    #Code to load in OIF
    def _4ods_loadOif(self,oifString):

        p = OifParseTreeMapping.OifParseTreeMapping()
        tree = p.parse(oifString)

        self.__tags = {}

        self.__4ods_processOifFile(tree[0])

    def __4ods_processOifFile(self,oifFile):

        self.__4ods_processObjectDefList(oifFile[0])

    def __4ods_processObjectDefList(self,objDefList):

        self.__4ods_processObjectDef(objDefList[0])
        if objDefList[1]:
            self.__4ods_processObjectDefList(objDefList[1])

    def __4ods_processObjectDef(self,objDef):

        if objDef[5]:
            raise "Physically Near to not supported"

        objTag = self.__4ods_processObjectTag(objDef[0])
        className = self.__4ods_processClassName(objDef[1])

        ps = string.split(className,'.')
        curMod = __import__(string.join(ps[:-1],'.'))
        for p in ps[1:]:
            curMod = getattr(curMod,p)

        inst = curMod()
        if objTag[:3] != 'tag':
            from Ft.Ods import Database
            db = Database.Database()
            db.bind(inst,string.replace(objTag,'$',' '))
        repoObj = self._dba.getRepositoryObject(inst.typeId)


        init = self.__4ods_processInitialization(objDef[3])

        for i in init:
            if type(i[1]) == type(()):
                func = getattr(inst,'form_%s'%i[0])
                for targetName in i[1]:
                    if self.__tags.has_key(targetName):
                        target = self.__tags[targetName]
                        apply(func,(target,))
            else:
                index = list(inst.tupleNames).index(i[0])
                if inst.tupleTypes[index] == Constants.Types.ENUMERATION:
                    setattr(inst,i[0],inst._4ods_getPythonClass(i[0]).lookup(i[1]))
                else:
                    setattr(inst,i[0],i[1])

        self.__tags[objTag] = inst
            

    def __4ods_processObjectTag(self,objTag):
        return objTag[0]

    def __4ods_processClassName(self,className):
        return self.__4ods_processScopedName(className[0])


    def __4ods_processScopedName(self,scopedName):
        name = ''
        if not scopedName[1]:
            #Simple Name
            name =  scopedName[0]
        elif not scopedName[2]:
            #Global scoped name
            name =  scopedName[1]
        else:
            #A name in a chain
            first = self.__4ods_processScopedName(scopedName[0])
            name =  first + '.' + scopedName[2]
        return name

    def __4ods_processInitialization(self,init):
        if init[0]['TYPE'] == 'ATTRIBUTE_LIST':
            return self.__4ods_processAttributeList(init[0])
        else:
            raise "Not Supported %s" % str(init[0])

    def __4ods_processAttributeList(self,attrList):
        attrs = [self.__4ods_processAttribute(attrList[0])]
        if attrList[2]:
            return attrs + self.__4ods_processAttributeList(attrList[2])
        return attrs

    def __4ods_processAttribute(self,attr):
        return (self.__4ods_processFieldName(attr[0]),self.__4ods_processValue(attr[1]))


    def __4ods_processCollectionValue(self,collVal):
        return self.__4ods_processValueList(collVal[1])

    def __4ods_processFieldName(self,fieldName):
        return fieldName[0]

    def __4ods_processValueList(self,valueList):
        values = self.__4ods_processValue(valueList[0])
        if valueList[2]:
            values = values + self.__4ods_processValueList(valueList[2])
        return values

    def __4ods_processValue(self,value):
        if value[0]['TYPE'] == 'LITERAL':
            return self.__4ods_processLiteral(value[0])
        elif value[0]['TYPE'] == 'COLLECTION_VALUE':
            return tuple(self.__4ods_processCollectionValue(value[0]))
        else:
            raise "Not supported %s" % str(value[0]['TYPE'])
                 
    def __4ods_processLiteral(self,literal):
        if literal[0]['TYPE'] == 'STRING_LITERAL':
            return str(self._4ods_unescapeQuotes(literal[0][0][1:-1]))
        elif literal[0]['TYPE'] == 'FLOAT_LITERAL':
            return float(literal[0][0])
        elif literal[0]['TYPE'] == 'INTEGER_LITERAL':
            return int(literal[0][0])
        elif literal[0]['TYPE'] == 'OBJECT_TAG':
            return (self.__4ods_processObjectTag(literal[0]),)
        else:
            raise "Not supported %s" % str(literal[0]['TYPE'])


    #Code to output OIF
    def _4ods_dumpOif(self,out):

        oids = self._dba.getAllObjectIds()


        for oid in oids:
            obj = self.getObject(oid)
            curTagName = self._4ods_generateTagName(obj)

            repoClass = self.getRepositoryObject(obj.typeId)

            #Get the actual class Name
            mods = string.split(obj.__class__.__module__,'.')
            if mods[0] == '__main__':
                import sys
                mods[0] = string.join(string.split(sys.argv[0],'.')[:-1],'.')
            pName = string.join(mods,'.') + '.' + obj.__class__.__name__            
            out.write("%s %s {" % (curTagName,pName))

            #Add Attributes
            gotOneFlag = 0
            tmpStr = ''
            for attrName in obj.readOnlyAttributes + obj.readWriteAttributes:
                if attrName == '_oid':
                    continue
                gotOneFlag = 1
                tmpStr = tmpStr + "%s %s," % (attrName,self.__4ods_getAttributeValue(repoClass.resolve(attrName),getattr(obj,attrName)))

            #Add Relationships
            for relName in obj.relationships:
                if hasattr(obj,relName):
                    relVal = self.__4ods_getRelationValue(repoClass.resolve(relName),getattr(obj,relName))
                    if relVal:
                        gotOneFlag = 1
                        tmpStr = tmpStr + "%s," % (relVal)
            if gotOneFlag:
                tmpStr = tmpStr[:-1]

            out.write(tmpStr)

            out.write('}\n')

        return


    def __4ods_getRelationValue(self,repo,val):

        if repo._4ods_getCardinality() == 'N':
            #This is a one to N
            #Build a list of tag names
            tns = []
            for oid in map(lambda x:x._4ods_getId(),val._4ods_getContents()):
                obj = self.getObject(oid)
                tn = self._4ods_generateTagName(obj)
                tns.append(tn)
            if len(tns) == 0:
                return ''
            rt = '%s { ' % repo.name
            for tn in tns:
                rt = rt + "%s," % tn
            rt = rt[:-1] + "}"
            return rt
        else:
            if not val:
                return None
            v = self._4ods_generateTagName(val)
            return "%s %s" % (repo.name,v)
            
            
            

    def __4ods_getAttributeValue(self,repo,val):
        from Ft.Ods.MetaData import RepositoryObject
        from Ft.Ods.MetaData import PrimitiveType

        if repo.type.meta_kind == RepositoryObject.MetaKind.mk_primitive_type:
            if repo.type.primitive_kind == PrimitiveType.PrimitiveKind.pk_double:
                return str(val)
            elif repo.type.primitive_kind == PrimitiveType.PrimitiveKind.pk_short:
                return str(val)
            elif repo.type.primitive_kind == PrimitiveType.PrimitiveKind.pk_ushort:
                return str(val)
            elif repo.type.primitive_kind == PrimitiveType.PrimitiveKind.pk_string:
                return '"' + self._4ods_escapeQuotes(val) + '"'
            else:
                raise "Unsupported Primitive Type %s" % repo.type.primitive_kind
        elif repo.type.meta_kind == RepositoryObject.MetaKind.mk_enumeration:
            #Hmmm, we have to do this as a string I guess?
            return '"' + val._n + '"'
        else:
            raise "Unsupported Attribute value %s" % str(repo.type)

    def _4ods_unescapeQuotes(self,val):
        return string.replace(val,'&#34;','"')
    def _4ods_escapeQuotes(self,val):
        return string.replace(val,'"','&#34;')

    def _4ods_generateTagName(self,val):
        name = self.isBound(val)
        if name:
            curTagName = string.replace(name,' ','$')
        else:
            curTagName = "tag%d"%val._4ods_getId()
        return curTagName

def pp(map):
    for k in map.keys():
        print k,
        print ' ',
    print

