############################################################################
#
# File Name: 		__init__.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/StorageManager/__init__.py/html
#
"""
Manages data storage
WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 2000 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

from Ft.Ods.StorageManager import Adapters
from Ft.Ods.StorageManager import QueryAdapter
from Ft.Ods.StorageManager import OifAdapter


import ExtentManager


class StorageManager(QueryAdapter.QueryAdapter,OifAdapter.OifAdapter):
    def __init__(self,dbName):
        QueryAdapter.QueryAdapter.__init__(self)
        OifAdapter.OifAdapter.__init__(self)
        self.__dbName = dbName

        #Create our managers
        self._dba = Adapters.g_driverModule.DbAdapter(self.__dbName)
        #Repo objects are managed through an adapter for now
        #It needs to cache anyways!!
        self.__repoManager = self._dba

        self.__eManager = ExtentManager.ExtentManager(self._dba)


    def collectGarbage(self):
        #self.__storageManager.collectGarbage()
        pass

    def newRepositoryObjectId(self,ro):
        return self.__repoManager.newRepositoryObjectId(ro)

    def newRepositoryObject(self,ro):
        self.__repoManager.newRepositoryObject(ro)
        return

    def newObjectId(self,o):
        self.__repoManager.newObjectId(o)
        return

    def newLiteralId(self,l):
        self.__repoManager.newLiteralId(l)
        return

    def newCollectionId(self,c):
        self.__repoManager.newCollectionId(c)
        return

    def newLiteral(self,l):
        self.__repoManager.newLiteral(l)
        return


    def writeRepositoryObject(self,ro):
        self._dba.writeRepositoryObject(ro)
        return

    def getRepositoryObject(self, id):
        return self._dba.getRepositoryObject(id)

    def deleteRepositoryObject(self, ro):
        self._dba.deleteRepositoryObject(ro)
        return

    def writeObject(self, o):
        self._dba.writeObject(o)
        return

    def getObject(self, id):
        return self._dba.getObject(id)

    def getLiteral(self, id):
        return self._dba.getLiteral(id)

    def getCollection(self, id):
        return self._dba.getCollection(id)

    def loadCollectionData(self,c,index):
        return self._dba.loadCollectionData(c,index)
        
        


    def writeLiteral(self, l):
        self._dba.writeLiteral(l)
        return

    def deleteObject(self,o):
        self._dba.deleteObject(o)
        return

    def deleteLiteral(self,l):
        self._dba.deleteLiteral(l)
        return


    def writeCollection(self,c):
        self._dba.writeCollection(c)
        return
        
    def deleteCollection(self, c):
        self._dba.deleteCollection(c)
        return


    def bind(self,obj,name):
        self.__eManager._4ods_checkName(name)
        return self._dba.bind(obj,name)

    def unbind(self,name):
        return self._dba.unbind(name)

    def lookup(self,name):
        ext = self.__eManager.lookup(name)
        if ext != None:
            return ext
        return self._dba.lookup(name)

    def loadExtent(self,id):
        return self._dba._4ods_loadExtent(id)

    def getExtentTuple(self,name):
        ext = self.__eManager.lookup(name)
        if ext == None:
            return []
        return ext._4ods_toTuple()

    def getAllBindings(self,object):
        return self.__eManager.getAllBindings(object) + self._dba.getAllBindings(object)

    def isBound(self,object):
        return self._dba.isBound(object)

    def _4ods_commit(self):
        self._dba.commit()
        self.__eManager._psuedo_del()
        return

    def _4ods_abort(self):
        self._dba.abort()
        self.__eManager._psuedo_del()
        return

    def _4ods_checkpoint(self):
        self._dba.checkpoint()



