#!/usr/bin/env python
############################################################################
#  
#  Copyright (c)2000 FourThought, Inc.
#  http://ftsuite.com/4ODS
#
############################################################################
#
# File Name: 		4odb_dig.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/Tools/4odb_dig.py.html
#


#A Tool for pretty printing a 4ODS database
import string
from Ft.Ods.StorageManager import Extent
from Ft.Ods.Collections import CollectionBase
from Ft.Ods import Object

def ShowBoundName(name,db):
    print '*'*60
    print 'Bound Name ', name
    print '*' * 60

    steps = string.split(name,'.')

    o = db.lookup(steps[0])

    o = ResolveSteps(o,steps[1:],db)

    if o == None:
        raise "Unknown Name %s" % name
    Show(o)


def ResolveSteps(o,steps,db):
    if len(steps) == 0:
        return o
    if isinstance(o,Extent.Extent):
        raise "Cannot step through a collection"

    o = getattr(o,steps[0])
    return ResolveSteps(o,steps[1:],db)


def Show(obj,indent=0):
    print '  ' * indent + "Class: ",str(obj.__class__)
    if isinstance(obj,Extent.Extent):
        ShowCollection(obj,indent)
        return
    ShowObject(obj,obj.__class__,indent)


def ShowObject(obj,klass,indent=0):

    for rwAttr in klass.readOnlyAttributes + klass.readWriteAttributes:
        print '  ' * indent + "  Attribute: " + rwAttr +  " =  " + str(getattr(obj,rwAttr))

    for rel in klass.relationships:
        if hasattr(obj,rel):
            target = getattr(obj,rel)
        else:
            target = None
        if target:
            if isinstance(target,CollectionBase.CollectionBase):
                targetStr = "Oids %s" % str(map(lambda x:x._4ods_getId(),target._4ods_getContents()))
            else:
                targetStr = "Oid: %d" % target._4ods_getOid()
        else:
            targetStr = "Nothing"
        print '  ' * indent + "  Relationship: " + rel +  " to " + targetStr



def ShowCollection(coll,indent):

    for c in coll._4ods_getContents():
        print c
        Show(c,indent+1)


def PrintUsage():
    usage_string = """4odb_dig.py dbName [boundPath]\n\tboundPath A name bound in the database Example Einstein.teaches.  All bound objects will be shown by default"""
    print usage_string

def Run(argv):

    import sys,string

    if len(argv) < 2:
        PrintUsage()
        sys.exit()


    dbName = argv[1]
    from Ft.Ods import Database

    db = Database.Database()
    db.open(dbName)
    tx = db.new()
    tx.begin()
    
    boundName = ''
    if len(argv) > 2:
        boundName = argv[2]

    if boundName:
        ShowBoundName(boundName,db)
    else:
        names = db.keys()
        for name in names:
            if name == 'ftods_types':
                continue
            ShowBoundName(name,db)


if __name__ == '__main__':

    import sys
    Run(sys.argv)
