#!/usr/bin/env python
############################################################################
#  
#  Copyright (c)2000 FourThought, Inc.
#  http://ftsuite.com/4ODS
#
############################################################################
#
# File Name: 		4odb_dig.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/Tools/4odb_dig.py.html
#




from Ft.Ods.MetaData import PrimitiveType
from Ft.Ods.MetaData import Class
from Ft.Ods.MetaData import Structure
from Ft.Ods.MetaData import Attribute
from Ft.Ods.MetaData import Repository
from Ft.Ods.MetaData import Member
from Ft.Ods.MetaData import Module
from Ft.Ods.MetaData import Relationship
from Ft.Ods.MetaData import Enumeration
from Ft.Ods.MetaData import Collection
from Ft.Ods.MetaData import Constant


def ShowRepository(repo):
    print '*' * 60
    print "Schema of the Repository"
    print '*' * 60
    ShowModule(repo,0)

def ShowModule(mod,indent):
    print '  ' * indent + "Module: ", mod.absolute_name()
    indent = indent + 1
    ShowMetaObject(mod,indent)
    ShowDefiningScope(mod,indent)

def ShowStructure(struct,indent):
    print '  ' * indent + "Structure: ", struct.absolute_name()
    indent = indent + 1
    print '  ' * indent + 'Fields:'
    for m in struct.fields._ftods_getContents():
        ShowMember(m,indent+1)
    if struct.exceptionResult:
        raise "Exception Result"
    ShowScopedType(struct,indent)

def ShowClass(klass,indent):
    print '  ' * indent + "Class: ", klass.absolute_name()
    indent = indent + 1
    exts = klass.extents._4ods_getContents()
    if len(exts):
        ext = exts[0]
    else:
        ext = None
    print '  ' * indent + "Extent: ", ext
    print '  ' * indent + "Keys: None"
    if klass.extender:
        print '  ' * indent + "Extender: ",klass.extender.absolute_name()
    else:
        print '  ' * indent + "Extender: None"
    print '  ' * indent + "Extensions:"
    ShowCollection(klass.extensions,indent+1)
    ShowInterface(klass,indent)


def ShowEnumeration(enum,indent):
    print '  ' * indent + "Enumeration: ", enum.absolute_name()
    indent = indent + 1
    print '  ' * indent + 'Elements:'
    for e in enum.elements._4ods_getContents():
        ShowConstant(e,indent+1)
    ShowScopedType(enum,indent)

def ShowConstant(const,indent):
    print '  ' * indent + "Constant: ", const.absolute_name()
    indent = indent + 1    
    if const.the_Value:
        print '  ' * indent + 'the_Value: ',const.the_Value.value()
    else:
        print '  ' * indent + 'the_Value: None'
    print '  ' * indent + 'type: ', const.type.absolute_name()
    print '  ' * indent + 'referenced_by:'
    ShowCollection(const.referenced_by,indent+1)
    if const.enumeration:
        print '  ' * indent + 'enumeration: ',const.enumeration.absolute_name()
    else:
        print '  ' * indent + 'enumeration: None'
    print '  ' * indent + 'value: ',const.value()
    ShowMetaObject(const,indent)

    
def ShowInterface(interface,indent):
    print '  ' * indent + "Inherits: None"
    print '  ' * indent + "Derives: None"
    ShowType(interface,indent)
    ShowDefiningScope(interface,indent)


def ShowAttribute(attr,indent):
    print '  ' * indent + "Attribute: ",attr.absolute_name()
    indent = indent + 1
    print '  ' * indent + "is_read_only: %s" % str(attr.is_read_only)
    ShowProperty(attr,indent)


def ShowRelationship(rel,indent):
    print '  ' * indent + "Relationship: ",rel.absolute_name()
    indent = indent + 1
    print '  ' * indent + 'traversal: ',rel.traversal.absolute_name()
    ShowProperty(rel,indent)

def ShowProperty(property,indent):
    print '  ' * indent + "type: ", property.type.absolute_name()
    ShowMetaObject(property,indent)

def ShowMember(member,indent):
    print '  ' * indent + "Member: ", member.absolute_name()
    indent = indent + 1
    print '  ' * indent + 'Structure Type to: ', member.structure_type.absolute_name()
    ShowSpecifier(member,indent)


def ShowUnknownMetaObject(mo,indent):
    if isinstance(mo,Class.Class):
        ShowClass(mo,indent)
    elif isinstance(mo,Structure.Structure):
        ShowStructure(mo,indent)
    elif isinstance(mo,Attribute.Attribute):
        ShowAttribute(mo,indent)
    elif isinstance(mo,Repository.Repository):
        ShowRepository(mo)
    elif isinstance(mo,Member.Member):
        ShowMember(mo,indent)
    elif isinstance(mo,Module.Module):
        ShowModule(mo,indent)
    elif isinstance(mo,Relationship.Relationship):
        ShowRelationship(mo,indent)
    elif isinstance(mo,Enumeration.Enumeration):
        ShowEnumeration(mo,indent)
    elif isinstance(mo,Constant.Constant):
        ShowConstant(mo,indent)
    else:
        raise mo

def ShowMetaObject(mo,indent):
    print '  ' * indent + 'Name: ', mo.name
    print '  ' * indent + 'Comment: ', mo.comment
    if mo.definedIn:
        print '  ' * indent + 'Defined In: %s id(%i)' %(mo.definedIn.absolute_name(),mo._4ods_getId())
    else:
        print '  ' * indent + 'Defined In: GLOBAL'


def ShowScopedType(st,indent):
    ShowType(st,indent)
    ShowScope(st,indent)

def ShowDefiningScope(ds,indent):
    print '  ' * indent + "Defines:"
    if len(ds.defines._4ods_getContents()) == 0:
        print '  '*(indent + 1) + 'None'
        return
    for d in ds.defines._4ods_getContents():
         ShowUnknownMetaObject(d,indent+1)  
    ShowScope(ds,indent)

def ShowSpecifier(spec,indent):
    print "  " * indent + "name: ", spec.name
    print "  " * indent + "type: ", spec.type.name
    print "  " * indent + "metaId: ", spec.metaId
    

def ShowTypes(db):
    print '*' * 60
    print "Types defined in the Repository"
    print '*' * 60
    types = db._ftods_types()
    for t in types:
        if isinstance(t,PrimitiveType.PrimitiveType):
            print "PrimitiveType: ",
            print t.name
        elif isinstance(t,Class.Class):
            print "Class: ",
            print t.name
        elif isinstance(t,Collection.Collection):
            print "Collection: ",
            print t.name
            print 'kind: %s' % (t._ftods_getPythonClass())
            print 'subtype: %s' % t.subtype.name
        else:
            raise "Unknown Type"

        ShowType(t,1)


def ShowType(t,indent):
    #indent = indent + 1
    print "  " * indent + "Of Collection:"
    ShowCollection(t.collections,indent + 1)
    print "  " * indent + "Of Specifier:"
    ShowCollection(t.specifiers,indent + 1)
    print "  " * indent + "Of Union:"
    ShowCollection(t.unions,indent + 1)
    print "  " * indent + "Of Operation:"
    ShowCollection(t.operations,indent + 1)
    print "  " * indent + "Of Property:"
    ShowCollection(t.properties,indent + 1)
    print "  " * indent + "Of constant:"
    ShowCollection(t.constants,indent + 1)
    print "  " * indent + "Of TypeDefs:"
    ShowCollection(t.type_defs,indent + 1)


def ShowScope(sc,indent):
    pass

def ShowCollection(coll,indent):
    if len(coll._4ods_getContents()) == 0:
        print '  ' * indent + "None"
        return
    for e in coll._4ods_getContents():
        print '  ' * indent + ((e and e.absolute_name()) or '??None??')

def PrintUsage():
    usage_string = """4odb_metadig.py [-o] [-t] dbName [repo object]\n\t-o: Dump Odl\n\t-t: Dump Types"""
    print usage_string

def Run(argv):
    import sys
    driverName = 'Postgres'

    import getopt
    optList, args = getopt.getopt(argv[1:],'ot',[])

    showOdl = 0
    showTypes = 0
    for opt in optList:
        if opt[0] == '-o':
            showOdl = 1
        elif opt[0] == '-t':
            showTypes = 1
        else:
            PrintUsage()
            sys.exit()
    if len(args) < 1:
        PrintUsage()
        sys.exit()


    dbName = args[0]
    from Ft.Ods import Database

    db = Database.Database()
    db.open(dbName)

    tx = db.new()
    tx.begin()


    if len(args) > 1:
        top = db.schema().resolve(args[1])
        if not top:
            raise "Unknown MetaData Object %s" %args[1]
    else:
        top = db.schema().resolve("ODLMetaObjects")



    if showOdl:
        print '*'*60
        print 'Repository Odl'
        print '*' * 60
        print top._4ods_getOdl()
    if showTypes:
        ShowTypes(db)

    ShowUnknownMetaObject(top,0)

    tx.abort()
    db.close()


if __name__ == '__main__':
    import sys
    Run(sys.argv)
