########################################################################
#
# File Name:   ParsedXPointer.py
#
# Docs:        http://docs.4suite.com/XPointer/ParsedXPointer.py.html
#
"""
A Parsed Token that represents a list of XPointers
WWW: http://4suite.com/XPointer        e-mail: support@4suite.com

Copyright (c) 2000 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

import types
from Ft.XPointer import XPtrContext, XPtrException
from xml.dom.ext import GetAllNs
from xml.xpath import ParsedToken

class ParsedXPointer(ParsedToken.ParsedToken):
    def __init__(self, xptrs=None):
        ParsedToken.ParsedToken.__init__(self, 'XPOINTER')
        if xptrs is None:
            self._xpointers = []
            self._length = 0
        else:
            self._xpointers = list(xptrs)
            self._length = len(xptrs)

    def append(self,xptr):
        self._xpointers.append(xptr)
        self._length = self._length + 1

    def select(self, document, contextNode, nss = None):
        nss = nss or {}
        
        nss.update(GetAllNs(document))
        context = XPtrContext.XPtrContext(document, 1, 1, contextNode, nss)
        for xptr in self._xpointers:
            set = xptr.evaluate(context)
            if len(set) >= 1:
                if len(set) == 1:
                    return set[0]
                print 'Not a singleton result'
                print set
                break
            raise XPtrException(XPtrException.SUB_RESOURCE_ERROR)
            

    def __getitem__(self, index):
        return self._xpointers[index]

    def __len__(self):
        return self._length

    def pprint(self, indent=''):
        print indent + str(self)
        for xptr in self._xpointers:
            xptr.pprint(indent + '  ')

    def __str__(self):
        result = '<XPointer at %x' % id(self)
        if self._xpointers:
            result = result + ': ' + repr(self)
        return result + '>'

    def __repr__(self):
        return reduce(lambda result, xptr:
                      result + 'xpointer(%s)' % repr(xptr),
                      self._xpointers,
                      ''
                      )
