########################################################################
#
# File Name:            __init__.py
#
# Documentation:        http://docs.4suite.org/4DOM/ext/reader/__init__.py.html
#
"""
The 4DOM reader module has routines for deserializing XML and HTML to DOM
WWW: http://4suite.org/4DOM         e-mail: support@4suite.org

Copyright (c) 2000 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

import string, urllib, urlparse, cStringIO, os
from xml.dom.ext import ReleaseNode

try:
    import codecs
    from types import UnicodeType
    encoder = codecs.lookup("utf-8")[0] # encode,decode,reader,writer
    def StrStream(st):
        if type(st) is UnicodeType:
            st = encoder(st)[0]
        return cStringIO.StringIO(st)
except ImportError:
    StrStream = lambda x: cStringIO.StringIO(x)

import string, urlparse, urllib, os


class BaseUriResolver:
    def normalize(self, uri, base=''):
        "Default action is to simply resolve to an absolute URL"
        uri = urlparse.urljoin(base, uri)
        (scheme, netloc, path, params, query, fragment) = urlparse.urlparse(uri)
        if path:
            paths = os.path.split(path)
            path = apply(os.path.join,paths[:-1])
        baseUri = urlparse.urlunparse((scheme, netloc, path, params, query, fragment))
        if baseUri and not baseUri[-1] == '/':
            baseUri = baseUri + '/'
        return uri, baseUri

    def resolve(self, uri, base=''):
        uri = self.normalize(uri, base)[0]
        #Hack cause urllib doesn't work well on Windows
        if os.path.exists(uri):
            stream = open(uri)
        else:
            stream = urllib.urlopen(uri)
        return stream


BASIC_RESOLVER = BaseUriResolver()

class Reader:
    def clone(self):
        """Used to create a new copy of this instance"""
        if hasattr(self,'__getinitargs__'):
            return apply(self.__class__,self.__getinitargs__())
        else:
            return self.__class__()

    def fromStream(self,stream, ownerDoc=None):
        """Create a DOM from a stream"""
        raise "NOT OVERIDDEN"

    def fromString(self, st, ownerDoc=None):
        """Create a DOM from a string"""
        #The defauilt implementaton
        stream = StrStream(st)
        rt = self.fromStream(stream, ownerDoc)
        stream.close()
        return rt

    def fromUri(self, uri, ownerDoc=None):
        #Hack cause urilib doesn't work well on Windows
        if os.path.exists(uri):
            stream = open(uri)
        else:
            stream = urllib.urlopen(uri)
        rt = self.fromStream(stream, ownerDoc)
        stream.close()
        return rt

    def releaseNode(self, node):
        "Free a DOM tree"
        node and ReleaseNode(node)

