#!/usr/bin/env python
########################################################################
#
# File Name:            4xslt.py
#
# Documentation:        http://docs.4suite.com/4XSLT/4xslt.py.html
#
"""
Command-line invokation of the 4XSLT processor
WWW: http://4suite.com/4XSLT        e-mail: support@4suite.com

Copyright (c) 1999 FourThought LLC, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

import re, string, os, sys, getopt, cStringIO, traceback
from xml.dom.Node import Node
import xml.dom.ext
from xml.xslt import XsltException
from xml.xslt.Processor import Processor

g_paramBindingPattern = re.compile(r"([\d\D_\.\-]*:?[\d\D_\.\-]+)=(.*)")

def Run(argv):
    usage = """
Usage:

%s [ -i ] [ -v ] [ -D <param_name>=<value>] [ -o <output_filename> ] souce_uri [stylesheet1, ... stylesheetN]
Options:
    -i    Ignore style-sheet processing instructions in the input file
    -v    Validate the input file as it is being parsed
    -D    Bind a top-level parameter, overriding any binding in the stylesheet
    -o    Specify a filename for the output.  This file will be overwritten
          if present.

Note: if you use "-" as the name of the source document, the source will
instead be read from standard input
    """ % argv[0]

    validate_flag = 0
    out_file = sys.stdout
    ignore_pis = 0
    top_level_params = {}
    command_line_error = 0
    stylesheets = []
    optlist, args = getopt.getopt(argv[1:], 'ivD:o:')
    
    if len(args) == 0:
        command_line_error = 1
    else:
        source = args[0]
    if len(args) > 1:
        stylesheets = args[1:]

    for op in optlist:
        if op[0] == "-v":
            validate_flag = 1
        elif op[0] == "-i":
            ignore_pis = 1
        elif op[0] == "-o":
            out_file = open(op[1], 'w')
        elif op[0] == "-D":
            match = g_paramBindingPattern.match(op[1])
            top_level_params[match.group(1)] = match.group(2)
        else:
            command_line_error = 1

    if command_line_error:
        print usage
        sys.exit(1)

    processor = Processor()
    import os
    from Ft.Lib import pDomlette
    BETA_DOMLETTE = os.environ.get("BETA_DOMLETTE")
    if BETA_DOMLETTE and not validate_flag:
        from Ft.Lib import cDomlette
        g_readerClass = cDomlette.RawExpatReader
        reader = cDomlette.RawExpatReader()
    elif validate_flag:
        reader = pDomlette.SaxReader(validate=1)
    else:
        reader = pDomlette.PyExpatReader()

    try:
        processor.setDocumentReader(reader)
        for sty in stylesheets:
            processor.appendStylesheetUri(sty)
        if source == '-':
            result = processor.runStream(sys.stdin, ignore_pis,
                                      topLevelParams=top_level_params)
        else:
            result = processor.runUri(source, ignore_pis,
                                      topLevelParams=top_level_params)
    except XsltException, e:
        s = cStringIO.StringIO()
        traceback.print_exc(1000, s)
        print s.getvalue()
        sys.stderr.write(str(e) + '\n')
        sys.exit(-1)
    out_file.write(result + '\n')
    out_file.close()


if __name__ == '__main__':
    import sys
    Run(sys.argv)
