#!/usr/bin/env python
#
# File Name:        File Name
#
# Author:           Mike Olson
#
# Creation Date:    1999-08-06
#
# Documentation:    http://docs.fourthought.com/file/name.html
#
#

def test(tester=None):
    if tester is None:
        import TestSuite
        tester = TestSuite.TestSuite()

    tester.startGroup('ParsedNodeTest')
    
    tester.startTest('Creating test environment')
    from xml.xpath import ParsedNodeTest
    from xml.xpath import XPath
    from xml.xpath import Context
    from xml.xpath.Boolean import Boolean,True,False
    import DomTree

    context = Context.Context(DomTree.ROOT,1,1,{},{'bar':'http://foo.com'})
    tests = [((XPath.WILDCARD_NAME, '*'), [(DomTree.ROOT, True)]),
             ((XPath.WILDCARD_NAME, 'bar:CHILD3'), [(DomTree.ROOT, False),
                                                    (DomTree.CHILD1, False),
                                                    (DomTree.CHILD3, True),
                                                    ]),
             ((XPath.WILDCARD_NAME, 'bar:*'), [(DomTree.ROOT, False),
                                               (DomTree.CHILD1, False),
                                               (DomTree.CHILD3, True),
                                               ]),
             ((XPath.NODE, ''), [(DomTree.ROOT, True),
                                 (DomTree.TEXT1, True),
                                 ]),
             ((XPath.TEXT, ''), [(DomTree.ROOT, False),
                                 (DomTree.TEXT1, True),
                                 ]),
             ((XPath.COMMENT, ''), [(DomTree.ROOT, False),
                                    (DomTree.COMMENT, True),
                                    ]),
             ((XPath.PROCESSING_INSTRUCTION,''), [(DomTree.ROOT, False),
                                                  (DomTree.PI, True),
                                                  ]),
             ((XPath.PROCESSING_INSTRUCTION,'xml-stylesheet'), [(DomTree.PI, True),
                                                                (DomTree.PI2, False),
                                                                ]),
             ]
                                                  
    tester.testDone()
    
    tester.startTest('Running node-tests')
    for args, nodeTests in tests:
        p = apply(ParsedNodeTest.ParsedNodeTest, args)
        for node,expected in nodeTests:
            # Use Booleans for display only
            result = Boolean(p.match(context, node))
            tester.compare(expected, result, 'Filter of "%s" against %s' % (
                repr(p),
                node.nodeName,
                ))
    tester.testDone()

    return tester.groupDone()

if __name__ == '__main__':
    import sys
    retval = test()
    sys.exit(retval)

