import test_harness

from Ft.Lib import ReaderBase
from Ft.Lib import pDomlette
from Ft.Lib import cDomlette
from xml.xslt import StylesheetReader
from xml.dom.ext.reader import Sax2

sheet_str = """<?xml version="1.0"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
version="1.0">
  <xsl:include href='addr_book1.xsl'/>

  <xsl:template match='PHONENUM'>
        <xsl:element name='TD'>
        <xsl:attribute name='ALIGN'>CENTER</xsl:attribute>
        <xsl:text>Type: </xsl:text><xsl:value-of select='@DESC'/>
            <xsl:apply-templates/>
        </xsl:element>
  </xsl:template>
</xsl:stylesheet>
"""

expected = """<HTML>
  <HEAD>
    <META HTTP-EQUIV='Content-Type' CONTENT='text/html; charset=UTF-8'>
    <TITLE>Address Book</TITLE>
  </HEAD>
  <BODY>
    <H1>Tabulate just the Names</H1>
    <TABLE>
      <TR>
        <TD ALIGN='CENTER'>
          <B>Pieter Aaron</B>
        </TD>
        <TD ALIGN='CENTER'>Type: Work404-555-1234</TD>
        <TD ALIGN='CENTER'>Type: Fax404-555-4321</TD>
        <TD ALIGN='CENTER'>Type: Pager404-555-5555</TD>
      </TR>
      <TR>
        <TD ALIGN='CENTER'>
          <B>Emeka Ndubuisi</B>
        </TD>
        <TD ALIGN='CENTER'>Type: Work767-555-7676</TD>
        <TD ALIGN='CENTER'>Type: Fax767-555-7642</TD>
        <TD ALIGN='CENTER'>Type: Pager800-SKY-PAGEx767676</TD>
      </TR>
      <TR>
        <TD ALIGN='CENTER'>
          <B>Vasia Zhugenev</B>
        </TD>
        <TD ALIGN='CENTER'>Type: Work000-987-6543</TD>
        <TD ALIGN='CENTER'>Type: Cell000-000-0000</TD>
      </TR>
    </TABLE>
  </BODY>
</HTML>"""

class GenericReader(ReaderBase.DomletteReader):

    def __init__(self,reader,allArgs):
        ReaderBase.DomletteReader.__init__(self)
        self._used = 0
        self._reader = reader
        self._allArgs = allArgs

    def releaseNode(self,node):
        r = pDomlette.PyExpatReader()
        r.releaseNode(node)

    def fromStream(self, stream, baseUri='', ownerDoc=None,stripElements=None):
        self._used = self._used + 1
        reader = self._reader()
        if self._allArgs:
            src = reader.fromStream(stream,baseUri=baseUri,ownerDoc=ownerDoc,stripElements=stripElements)
        else:
            src = reader.fromStream(stream,ownerDoc=ownerDoc)
        
        if not ownerDoc:
            sty = StylesheetReader.FromDocument(src,baseUri = baseUri,stylesheetReader = GenericReader(self._reader,self._allArgs))
        else:
            sty = ownerDoc.createDocumentFragment()
            sty.__dict__['baseUri'] = baseUri
            StylesheetReader.DomConvert(src.firstChild,sty,ownerDoc,[],{},0)

        return sty


readers = [('pDomlette.PyExpat Reader',GenericReader(pDomlette.PyExpatReader,1)),
           ('pDomlette.Sax Reader',GenericReader(pDomlette.SaxReader,1)),
           ('cDomlette.RawExpat Reader',GenericReader(cDomlette.RawExpatReader,1)),
           ('Sax2 Reader',GenericReader(Sax2.Reader,0)),
           ]


def Test(tester):


    tester.startGroup("Include, default Reader")
    source = test_harness.FileInfo(uri="addr_book1.xml")
    sty = test_harness.FileInfo(string=sheet_str)
    xtest = test_harness.XsltTester(source,
                                    [sty],
                                    expected,
                                    )

    res = xtest.test(tester)
    tester.groupDone()
    if not res or xtest.quick:
        return res

    for name,reader in readers:
        tester.startGroup("Include, " + name)
        source = test_harness.FileInfo(uri="addr_book1.xml")
        sty = test_harness.FileInfo(string=sheet_str)
        xtest = test_harness.XsltTester(source,
                                        [sty],
                                        expected,
                                        stylesheetReader = reader
                                        )
        
        res = xtest.test(tester)
        
        tester.startTest("Number of Times Called")
        tester.testResults(xtest.quick and 1 or 6,reader._used,done=0)
        tester.testDone()
        tester.groupDone()


        if not res:
            return res

    return res


    processor.appendStylesheetString(sheet_str)
    result = processor.runUri('addr_book1.xml', ignorePis=1)
    if result != expected:
        raise Exception('Failed.  Expected\n"""%s"""\ngot\n"""%s"""'%(expected, result))
    print '*** Result ***'
    print result



if __name__ == '__main__':
    from Ft.Lib import TestSuite
    tester = TestSuite.TestSuite(1,1)
    Test(tester)




